# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys
import logging
import mock
import unittest
from collections import namedtuple

from azure.cli.core import AzCommandsLoader, MainCommandsLoader
from azure.cli.core.commands import ExtensionCommandSource
from azure.cli.core.extension import EXTENSIONS_MOD_PREFIX
from azure.cli.testsdk import TestCli

from knack.arguments import CLICommandArgument, CLIArgumentType


def _prepare_test_commands_loader(loader_cls, cli_ctx, command):
    loader = loader_cls(cli_ctx)
    loader.cli_ctx.invocation = mock.MagicMock()
    loader.cli_ctx.invocation.commands_loader = loader
    loader.command_name = command
    loader.load_command_table(None)
    loader.load_arguments(loader.command_name)
    loader._update_command_definitions()
    return loader


class TestCommandRegistration(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # Ensure initialization has occurred correctly
        logging.basicConfig(level=logging.DEBUG)

    @classmethod
    def tearDownClass(cls):
        logging.shutdown()

    @staticmethod
    def sample_vm_get(resource_group_name, vm_name, opt_param=None, expand=None, custom_headers=None, raw=False,
                      **operation_config):
        """
        The operation to get a virtual machine.

        :param resource_group_name: The name of the resource group.
        :type resource_group_name: str
        :param vm_name: The name of the virtual machine.
        :type vm_name: str
        :param opt_param: Used to verify reflection correctly
        identifies optional params.
        :type opt_param: object
        :param expand: The expand expression to apply on the operation.
        :type expand: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: VirtualMachine
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        pass

    def test_argument(self):

        class TestCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(TestCommandsLoader, self).load_command_table(args)
                with self.command_group('test register', operations_tmpl='{}#TestCommandRegistration.{{}}'.format(__name__)) as g:
                    g.command('sample-vm-get', 'sample_vm_get')
                return self.command_table

            def load_arguments(self, command):
                super(TestCommandsLoader, self).load_arguments(command)
                with self.argument_context('test register sample-vm-get') as c:
                    c.argument('vm_name', options_list=('--wonky-name', '-n'), metavar='VMNAME', help='Completely WONKY name...', required=False)

        cli = TestCli(commands_loader_cls=TestCommandsLoader)
        command = 'test register sample-vm-get'
        loader = _prepare_test_commands_loader(TestCommandsLoader, cli, command)

        command_metadata = loader.command_table[command]
        self.assertEqual(len(loader.command_table), 1,
                         'We expect exactly one command in the command table')
        self.assertEqual(len(command_metadata.arguments), 4, 'We expected exactly 4 arguments')
        some_expected_arguments = {
            'resource_group_name': CLIArgumentType(dest='resource_group_name', required=True),
            'vm_name': CLIArgumentType(dest='vm_name', required=False),
        }
        for probe in some_expected_arguments:
            existing = next(arg for arg in command_metadata.arguments if arg == probe)
            self.assertDictContainsSubset(some_expected_arguments[existing].settings,
                                          command_metadata.arguments[existing].options)
        self.assertEqual(command_metadata.arguments['vm_name'].options_list, ('--wonky-name', '-n'))

    def test_register_command(self):

        class TestCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(TestCommandsLoader, self).load_command_table(args)
                with self.command_group('test command', operations_tmpl='{}#TestCommandRegistration.{{}}'.format(__name__)) as g:
                    g.command('sample-vm-get', 'sample_vm_get')
                return self.command_table

            def load_arguments(self, command):
                super(TestCommandsLoader, self).load_arguments(command)
                with self.argument_context('test register sample-vm-get') as c:
                    c.argument('vm_name', options_list=('--wonky-name', '-n'), metavar='VMNAME', help='Completely WONKY name...', required=False)

        cli = TestCli(commands_loader_cls=TestCommandsLoader)
        command = 'test command sample-vm-get'
        loader = _prepare_test_commands_loader(TestCommandsLoader, cli, command)

        self.assertEqual(len(loader.command_table), 1,
                         'We expect exactly one command in the command table')
        command_metadata = loader.command_table['test command sample-vm-get']
        self.assertEqual(len(command_metadata.arguments), 4, 'We expected exactly 4 arguments')
        self.assertTrue(command_metadata.command_source is None)

        some_expected_arguments = {
            'resource_group_name': CLIArgumentType(dest='resource_group_name',
                                                   required=True,
                                                   help='The name of the resource group.'),
            'vm_name': CLIArgumentType(dest='vm_name',
                                       required=True,
                                       help='The name of the virtual machine.'),
            'opt_param': CLIArgumentType(required=False,
                                         help='Used to verify reflection correctly identifies optional params.'),  # pylint: disable=line-too-long
            'expand': CLIArgumentType(required=False,
                                      help='The expand expression to apply on the operation.')
        }

        for probe in some_expected_arguments:
            existing = next(arg for arg in command_metadata.arguments if arg == probe)
            self.assertDictContainsSubset(some_expected_arguments[existing].settings,
                                          command_metadata.arguments[existing].options)
        self.assertEqual(command_metadata.arguments['resource_group_name'].options_list,
                         ['--resource-group-name'])

    def _mock_import_lib(_):
        mock_obj = mock.MagicMock()
        mock_obj.__path__ = __name__
        return mock_obj

    def _mock_iter_modules(_):
        return [(None, __name__, None)]

    def _mock_extension_modname(ext_name, ext_dir):
        return ext_name

    def _mock_get_extensions():
        MockExtension = namedtuple('Extension', ['name', 'preview'])
        return [MockExtension(name=__name__ + '.ExtCommandsLoader', preview=False),
                MockExtension(name=__name__ + '.Ext2CommandsLoader', preview=False)]

    def _mock_load_command_loader(loader, args, name, prefix):

        class TestCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(TestCommandsLoader, self).load_command_table(args)
                with self.command_group('hello', operations_tmpl='{}#TestCommandRegistration.{{}}'.format(__name__)) as g:
                    g.command('world', 'sample_vm_get')
                return self.command_table

        # A command from an extension
        class ExtCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(ExtCommandsLoader, self).load_command_table(args)
                with self.command_group('hello', operations_tmpl='{}#TestCommandRegistration.{{}}'.format(__name__)) as g:
                    g.command('noodle', 'sample_vm_get')
                return self.command_table

        # A command from an extension that overrides the original command
        class Ext2CommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(Ext2CommandsLoader, self).load_command_table(args)
                with self.command_group('hello', operations_tmpl='{}#TestCommandRegistration.{{}}'.format(__name__)) as g:
                    g.command('world', 'sample_vm_get')
                return self.command_table

        if prefix == 'azure.cli.command_modules.':
            command_loaders = {'TestCommandsLoader': TestCommandsLoader}
        else:
            command_loaders = {'ExtCommandsLoader': ExtCommandsLoader, 'Ext2CommandsLoader': Ext2CommandsLoader}

        module_command_table = {}
        for _, loader_cls in command_loaders.items():
            command_loader = loader_cls(cli_ctx=loader.cli_ctx)
            command_table = command_loader.load_command_table(args)
            if command_table:
                module_command_table.update(command_table)
                loader.loaders.append(command_loader)  # this will be used later by the load_arguments method
        return module_command_table

    @mock.patch('importlib.import_module', _mock_import_lib)
    @mock.patch('pkgutil.iter_modules', _mock_iter_modules)
    @mock.patch('azure.cli.core.commands._load_command_loader', _mock_load_command_loader)
    @mock.patch('azure.cli.core.extension.get_extension_modname', _mock_extension_modname)
    @mock.patch('azure.cli.core.extension.get_extensions', _mock_get_extensions)
    def test_register_command_from_extension(self):

        from azure.cli.core.commands import _load_command_loader
        cli = TestCli()
        main_loader = MainCommandsLoader(cli)
        cli.loader = main_loader

        cmd_tbl = cli.loader.load_command_table(None)
        ext1 = cmd_tbl['hello noodle']
        ext2 = cmd_tbl['hello world']

        self.assertTrue(isinstance(ext1.command_source, ExtensionCommandSource))
        self.assertFalse(ext1.command_source.overrides_command)

        self.assertTrue(isinstance(ext2.command_source, ExtensionCommandSource))
        self.assertTrue(ext2.command_source.overrides_command)

    def test_argument_with_overrides(self):

        global_vm_name_type = CLIArgumentType(
            options_list=('--foo', '-f'), metavar='FOO', help='foo help'
        )
        derived_vm_name_type = CLIArgumentType(base_type=global_vm_name_type,
                                               help='first modification')

        class TestCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(TestCommandsLoader, self).load_command_table(args)
                with self.command_group('test', operations_tmpl='{}#TestCommandRegistration.{{}}'.format(__name__)) as g:
                    g.command('vm-get', 'sample_vm_get')
                    g.command('command vm-get-1', 'sample_vm_get')
                    g.command('command vm-get-2', 'sample_vm_get')
                return self.command_table

            def load_arguments(self, command):
                super(TestCommandsLoader, self).load_arguments(command)
                with self.argument_context('test') as c:
                    c.argument('vm_name', global_vm_name_type)

                with self.argument_context('test command') as c:
                    c.argument('vm_name', derived_vm_name_type)

                with self.argument_context('test command vm-get-2') as c:
                    c.argument('vm_name', derived_vm_name_type, help='second modification')

        cli = TestCli(commands_loader_cls=TestCommandsLoader)
        loader = _prepare_test_commands_loader(TestCommandsLoader, cli, 'test vm-get')
        self.assertEqual(len(loader.command_table), 3,
                         'We expect exactly three commands in the command table')
        command1 = loader.command_table['test vm-get'].arguments['vm_name']
        self.assertTrue(command1.options['help'] == 'foo help')

        loader = _prepare_test_commands_loader(TestCommandsLoader, cli, 'test command vm-get-1')
        command2 = loader.command_table['test command vm-get-1'].arguments['vm_name']
        self.assertTrue(command2.options['help'] == 'first modification')

        loader = _prepare_test_commands_loader(TestCommandsLoader, cli, 'test command vm-get-2')
        command3 = loader.command_table['test command vm-get-2'].arguments['vm_name']
        self.assertTrue(command3.options['help'] == 'second modification')

    def test_extra_argument(self):

        class TestCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(TestCommandsLoader, self).load_command_table(args)
                with self.command_group('test command', operations_tmpl='{}#TestCommandRegistration.{{}}'.format(__name__)) as g:
                    g.command('sample-vm-get', 'sample_vm_get')
                return self.command_table

            def load_arguments(self, command):
                super(TestCommandsLoader, self).load_arguments(command)
                with self.argument_context('test command sample-vm-get') as c:
                    c.extra('added_param', options_list=['--added-param'], metavar='ADDED', help='Just added this right now!', required=True)

        cli = TestCli(commands_loader_cls=TestCommandsLoader)
        command = 'test command sample-vm-get'
        loader = _prepare_test_commands_loader(TestCommandsLoader, cli, command)

        self.assertEqual(len(loader.command_table), 1,
                         'We expect exactly one command in the command table')
        command_metadata = loader.command_table['test command sample-vm-get']
        self.assertEqual(len(command_metadata.arguments), 5, 'We expected exactly 5 arguments')

        some_expected_arguments = {
            'added_param': CLIArgumentType(dest='added_param', required=True)
        }

        for probe in some_expected_arguments:
            existing = next(arg for arg in command_metadata.arguments if arg == probe)
            self.assertDictContainsSubset(some_expected_arguments[existing].settings,
                                          command_metadata.arguments[existing].options)

    def test_command_build_argument_help_text(self):

        def sample_sdk_method_with_weird_docstring(param_a, param_b, param_c):  # pylint: disable=unused-argument
            """
            An operation with nothing good.

            :param dict param_a:
            :param param_b: The name
            of
            nothing.
            :param param_c: The name
            of

            nothing2.
            """

        class TestCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(TestCommandsLoader, self).load_command_table(args)
                with self.command_group('test command', operations_tmpl='{}#{{}}'.format(__name__)) as g:
                    g.command('foo', sample_sdk_method_with_weird_docstring.__name__)
                return self.command_table

        setattr(sys.modules[__name__], sample_sdk_method_with_weird_docstring.__name__,
                sample_sdk_method_with_weird_docstring)  # pylint: disable=line-too-long

        cli = TestCli(commands_loader_cls=TestCommandsLoader)
        command = 'test command foo'
        loader = _prepare_test_commands_loader(TestCommandsLoader, cli, command)

        command_metadata = loader.command_table[command]
        self.assertEqual(len(command_metadata.arguments), 3, 'We expected exactly 3 arguments')
        some_expected_arguments = {
            'param_a': CLIArgumentType(dest='param_a', required=True, help=''),
            'param_b': CLIArgumentType(dest='param_b', required=True, help='The name of nothing.'),
            'param_c': CLIArgumentType(dest='param_c', required=True, help='The name of nothing2.')
        }

        for probe in some_expected_arguments:
            existing = next(arg for arg in command_metadata.arguments if arg == probe)
            self.assertDictContainsSubset(some_expected_arguments[existing].settings,
                                          command_metadata.arguments[existing].options)

    def test_override_existing_option_string(self):
        arg = CLIArgumentType(options_list=('--funky', '-f'))
        updated_options_list = ('--something-else', '-s')
        arg.update(options_list=updated_options_list, validator=lambda: (), completer=lambda: ())
        self.assertEqual(arg.settings['options_list'], updated_options_list)
        self.assertIsNotNone(arg.settings['validator'])
        self.assertIsNotNone(arg.settings['completer'])

    def test_dont_override_existing_option_string(self):
        existing_options_list = ('--something-else', '-s')
        arg = CLIArgumentType(options_list=existing_options_list)
        arg.update()
        self.assertEqual(arg.settings['options_list'], existing_options_list)

    def test_override_remove_validator(self):
        existing_options_list = ('--something-else', '-s')
        arg = CLIArgumentType(options_list=existing_options_list,
                              validator=lambda *args, **kwargs: ())
        arg.update(validator=None)
        self.assertIsNone(arg.settings['validator'])

    def test_override_using_argument(self):
        def sample_sdk_method(param_a):  # pylint: disable=unused-argument
            pass

        def test_validator_completer():
            pass

        class TestCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super(TestCommandsLoader, self).load_command_table(args)
                with self.command_group('override_using_register_cli_argument', operations_tmpl='{}#{{}}'.format(__name__)) as g:
                    g.command('foo', 'sample_sdk_method')
                return self.command_table

            def load_arguments(self, command):
                super(TestCommandsLoader, self).load_arguments(command)
                with self.argument_context('override_using_register_cli_argument') as c:
                    c.argument('param_a', options_list=('--overridden', '-r'), required=False,
                               validator=test_validator_completer, completer=test_validator_completer)

        setattr(sys.modules[__name__], sample_sdk_method.__name__, sample_sdk_method)
        cli = TestCli(commands_loader_cls=TestCommandsLoader)
        command = 'override_using_register_cli_argument foo'
        loader = _prepare_test_commands_loader(TestCommandsLoader, cli, command)

        command_metadata = loader.command_table[command]
        self.assertEqual(len(command_metadata.arguments), 1, 'We expected exactly 1 arguments')

        actual_arg = command_metadata.arguments['param_a']
        self.assertEqual(actual_arg.options_list, ('--overridden', '-r'))
        self.assertEqual(actual_arg.validator, test_validator_completer)
        self.assertEqual(actual_arg.completer, test_validator_completer)
        self.assertFalse(actual_arg.options['required'])

    def test_override_argtype_with_argtype(self):
        existing_options_list = ('--default', '-d')
        arg = CLIArgumentType(options_list=existing_options_list, validator=None, completer='base',
                              help='base', required=True)
        overriding_argtype = CLIArgumentType(options_list=('--overridden',), validator='overridden',
                                             completer=None, overrides=arg, help='overridden',
                                             required=CLIArgumentType.REMOVE)
        self.assertEqual(overriding_argtype.settings['validator'], 'overridden')
        self.assertEqual(overriding_argtype.settings['completer'], None)
        self.assertEqual(overriding_argtype.settings['options_list'], ('--overridden',))
        self.assertEqual(overriding_argtype.settings['help'], 'overridden')
        self.assertEqual(overriding_argtype.settings['required'], CLIArgumentType.REMOVE)

        cmd_arg = CLICommandArgument(dest='whatever', argtype=overriding_argtype,
                                     help=CLIArgumentType.REMOVE)
        self.assertFalse('required' in cmd_arg.options)
        self.assertFalse('help' in cmd_arg.options)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import argparse
import unittest
from azure.cli.command_modules.cdn._params import OriginType


class OriginTypeTests(unittest.TestCase):

    def setUp(self):
        self.subject = OriginType(None, None)

    def _check_values(self, origin, host_name, http_port, https_port):
        self.assertEqual(origin.host_name, host_name)
        self.assertEqual(origin.http_port, http_port)
        self.assertEqual(origin.https_port, https_port)

    def test_origin_type_just_domain(self):
        values = ['www.domain.com']
        self._check_values(self.subject.get_origin(values, None), values[0], 80, 443)

    def test_origin_type_with_http_port(self):
        values = ['www.domain.com', 300]
        self._check_values(self.subject.get_origin(values, None), values[0], 300, 443)

    def test_origin_type_with_http_and_https_port(self):
        values = ['www.domain.com', 300, 5000]
        self._check_values(self.subject.get_origin(values, None), values[0], 300, 5000)

    def test_origin_too_many_values(self):
        values = ['www.domain.com', 300, 5000, 'bad']
        with self.assertRaises(argparse.ArgumentError):
            self.subject.get_origin(values, None)

    def test_origin_too_few_values(self):
        values = []
        with self.assertRaises(argparse.ArgumentError):
            self.subject.get_origin(values, None)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

try:
    import unittest.mock as mock
except ImportError:
    import mock

from knack.util import CLIError

from azure.cli.command_modules.monitor.operations.autoscale_settings import scaffold_autoscale_settings_parameters


class FilterBuilderTests(unittest.TestCase):
    def test_scaffold_autoscale_settings_parameters(self):
        template = scaffold_autoscale_settings_parameters(None)
        self.assertTrue(template)
        self.assertTrue(isinstance(template, dict))


def _mock_get_subscription_id(_):
    return '00000000-0000-0000-0000-000000000000'


class MonitorNameOrIdTest(unittest.TestCase):
    def _build_namespace(self, name_or_id=None, resource_group=None, provider_namespace=None, parent=None,
                         resource_type=None):
        from argparse import Namespace
        ns = Namespace()
        ns.name_or_id = name_or_id
        ns.resource_group_name = resource_group
        ns.namespace = provider_namespace
        ns.parent = parent
        ns.resource_type = resource_type
        return ns

    @mock.patch('azure.cli.core.commands.client_factory.get_subscription_id', _mock_get_subscription_id)
    def test_monitor_resource_id(self):
        from azure.cli.command_modules.monitor.validators import get_target_resource_validator
        from azure.cli.testsdk import TestCli

        cmd = mock.MagicMock()
        cmd.cli_ctx = TestCli()
        validator = get_target_resource_validator('name_or_id', True)

        id = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/my-rg/providers/Microsoft.Compute/' \
             'virtualMachines/vm1'

        # must supply name or ID
        ns = self._build_namespace()
        with self.assertRaises(CLIError):
            validator(cmd, ns)

        # must only supply ID or name parameters
        ns = self._build_namespace(id, 'my-rg', 'blahblah', 'stuff')
        with self.assertRaises(CLIError):
            validator(cmd, ns)

        # error on invalid ID
        ns = self._build_namespace('bad-id')
        with self.assertRaises(CLIError):
            validator(cmd, ns)

        # allow Provider/Type syntax (same as resource commands)
        ns = self._build_namespace('vm1', 'my-rg', None, None, 'Microsoft.Compute/virtualMachines')
        validator(cmd, ns)
        self.assertEqual(ns.name_or_id, id)

        # allow Provider and Type separate
        ns = self._build_namespace('vm1', 'my-rg', 'Microsoft.Compute', None, 'virtualMachines')
        validator(cmd, ns)
        self.assertEqual(ns.name_or_id, id)

        # verify works with parent
        id = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/my-rg/providers/Microsoft.Compute/' \
             'fakeType/type1/anotherFakeType/type2/virtualMachines/vm1'
        ns = self._build_namespace('vm1', 'my-rg', 'Microsoft.Compute', 'fakeType/type1/anotherFakeType/type2',
                                   'virtualMachines')
        validator(cmd, ns)
        self.assertEqual(ns.name_or_id, id)

        # verify extra parameters are removed
        self.assertFalse(hasattr(ns, 'resource_name'))
        self.assertFalse(hasattr(ns, 'namespace'))
        self.assertFalse(hasattr(ns, 'parent'))
        self.assertFalse(hasattr(ns, 'resource_type'))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest
import mock

from azure.cli.testsdk import TestCli
from azure.cli.command_modules.interactive.azclishell.configuration import Configuration
from azure.cli.command_modules.interactive.azclishell.app import AzInteractiveShell

from prompt_toolkit.document import Document


TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))


class CompletionTest(unittest.TestCase):
    def __init__(self, methodName):
        super(CompletionTest, self).__init__(methodName)
        with mock.patch.object(Configuration, 'get_help_files', lambda _: 'help_dump_test.json'):
            with mock.patch.object(Configuration, 'get_config_dir', lambda _: TEST_DIR):
                shell_ctx = AzInteractiveShell(TestCli(), None)
                self.completer = shell_ctx.completer
                self.shell_ctx = shell_ctx

    # tests the completion generator
    def verify_completions(self, generated_completions, expected_completions, start_position,
                           all_completions_expected=True, unexpected_completions=None):
        for completion in generated_completions:
            if all_completions_expected:
                self.assertIn(completion.text, expected_completions)
                expected_completions.remove(completion.text)
            else:
                expected_completions.discard(completion.text)
            if unexpected_completions:
                self.assertNotIn(completion.text, unexpected_completions)

            self.assertEqual(completion.start_position, start_position)
        self.assertFalse(expected_completions)

    def test_command_completion(self):
        # initial completions
        doc = Document(u' ')
        gen = self.completer.get_completions(doc, None)
        completions = set(['exit', 'quit', 'storage', 'vm', 'vmss'])
        self.verify_completions(gen, completions, 0)

        # start command
        doc = Document(u's')
        gen = self.completer.get_completions(doc, None)
        completions = set(['storage'])
        self.verify_completions(gen, completions, -1)

        # test spaces
        doc = Document(u'  storage    a')
        gen = self.completer.get_completions(doc, None)
        completions = set(['account'])
        self.verify_completions(gen, completions, -1)

        # completer traverses command-tree
        doc = Document(u'storage account ')
        gen = self.completer.get_completions(doc, None)
        completions = set(['create', 'check-name'])
        self.verify_completions(gen, completions, 0)

        # test completed command still shows completion
        doc = Document(u'vm')
        gen = self.completer.get_completions(doc, None)
        completions = set(['vm', 'vmss'])
        self.verify_completions(gen, completions, -2)

        # test unrecognized command does not complete
        doc = Document(u'vm group ')
        gen = self.completer.get_completions(doc, None)
        completions = set()
        self.verify_completions(gen, completions, 0)

    def test_param_completion(self):
        # 'az -h'
        doc = Document(u'-')
        gen = self.completer.get_completions(doc, None)
        completions = set(['-h'])
        self.verify_completions(gen, completions, -1)

        # 'az --help'
        doc = Document(u'--')
        gen = self.completer.get_completions(doc, None)
        completions = set(['--help'])
        self.verify_completions(gen, completions, -2)

        # 'az storage account -h'
        doc = Document(u'storage account -')
        gen = self.completer.get_completions(doc, None)
        completions = set(['-h'])
        self.verify_completions(gen, completions, -1)

        # test just completed command, first param completions
        doc = Document(u'storage account create ')
        gen = self.completer.get_completions(doc, None)
        expected = set(['--name', '--resource-group', '--access-tier', '--sku', '--tags'])
        not_expected = set(['-n', '-h', '-g'])
        self.verify_completions(gen, expected, 0, all_completions_expected=False, unexpected_completions=not_expected)

        # test single dash aliased params
        doc = Document(u'storage account create -')
        gen = self.completer.get_completions(doc, None)
        completions = set(['-h', '-o', '-g', '-l', '-n'])
        self.verify_completions(gen, completions, -1)

        # test params that are substrings of one another
        doc = Document(u'vmss create --subnet')
        gen = self.completer.get_completions(doc, None)
        completions = set(['--subnet', '--subnet-address-prefix'])
        self.verify_completions(gen, completions, -8)

        # test params with no help
        doc = Document(u'vmss create --upgrade-policy-mo')
        gen = self.completer.get_completions(doc, None)
        completions = set(['--upgrade-policy-mode'])
        self.verify_completions(gen, completions, -19)

        # test duplicated parameters
        expected = set()
        doc = Document(u'vmss create --name Bob --n')
        gen = self.completer.get_completions(doc, None)
        not_expected = set(['--name'])
        self.verify_completions(gen, expected, -3, all_completions_expected=False, unexpected_completions=not_expected)

        # test duplicated parameter alias
        doc = Document(u'vmss create --name Bob -')
        gen = self.completer.get_completions(doc, None)
        not_expected = set(['-n'])
        self.verify_completions(gen, expected, -1, all_completions_expected=False, unexpected_completions=not_expected)

        # test displayed help
        doc = Document(u'vm create -g')
        gen = self.completer.get_completions(doc, None)
        completion = next(gen)
        self.assertEqual(completion.text, '-g')
        self.assertIn('Name of resource group', completion._display_meta)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=too-many-statements

from azure.cli.testsdk import ResourceGroupPreparer, ScenarioTest


class IoTHubTest(ScenarioTest):

    @ResourceGroupPreparer()
    def test_iot_hub(self, resource_group, resource_group_location):
        device_id_1 = 'test-device-1'
        device_id_2 = 'test-device-2'
        hub = 'iot-hub-for-test'
        rg = resource_group
        location = resource_group_location

        # Test 'az iot hub create'
        self.cmd('iot hub create -n {0} -g {1} --sku S1 --partition-count 4'.format(hub, rg),
                 checks=[self.check('resourceGroup', rg),
                         self.check('location', location),
                         self.check('name', hub),
                         self.check('sku.name', 'S1'),
                         self.check('properties.eventHubEndpoints.events.partitionCount', '4')])

        # Test 'az iot hub show-connection-string'
        conn_str_pattern = r'^HostName={0}\.azure-devices\.net;SharedAccessKeyName=iothubowner;SharedAccessKey='.format(
            hub)
        self.cmd('iot hub show-connection-string -n {0}'.format(hub), checks=[
            self.check_pattern('connectionString', conn_str_pattern)
        ])

        self.cmd('iot hub show-connection-string -n {0} -g {1}'.format(hub, rg), checks=[
            self.check('length(@)', 1),
            self.check_pattern('connectionString', conn_str_pattern)
        ])

        # Test 'az iot hub update'
        property_to_update = 'properties.operationsMonitoringProperties.events.DeviceTelemetry'
        updated_value = 'Error, Information'
        self.cmd('iot hub update -n {0} --set {1}="{2}"'.format(hub, property_to_update, updated_value),
                 checks=[self.check('resourceGroup', rg),
                         self.check('location', location),
                         self.check('name', hub),
                         self.check('sku.name', 'S1'),
                         self.check(property_to_update, updated_value)])

        # Test 'az iot hub show'
        self.cmd('iot hub show -n {0}'.format(hub), checks=[
            self.check('resourceGroup', rg),
            self.check('location', location),
            self.check('name', hub),
            self.check('sku.name', 'S1'),
            self.check(property_to_update, updated_value)
        ])

        # Test 'az iot hub list'
        self.cmd('iot hub list -g {0}'.format(rg), checks=[
            self.check('length([*])', 1),
            self.check('[0].resourceGroup', rg),
            self.check('[0].location', location),
            self.check('[0].name', hub),
            self.check('[0].sku.name', 'S1')
        ])

        # Test 'az iot hub sku list'
        self.cmd('iot hub list-skus -n {0}'.format(hub), checks=[
            self.check('length([*])', 3),
            self.check('[0].sku.name', 'S1'),
            self.check('[1].sku.name', 'S2'),
            self.check('[2].sku.name', 'S3')
        ])

        # Test 'az iot hub policy create'
        policy_name = 'test_policy'
        permissions = 'RegistryWrite ServiceConnect DeviceConnect'
        self.cmd('iot hub policy create --hub-name {0} -n {1} --permissions {2}'.format(hub, policy_name, permissions),
                 checks=self.is_empty())

        # Test 'az iot hub policy list'
        self.cmd('iot hub policy list --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 6)
        ])

        # Test 'az iot hub policy show'
        self.cmd('iot hub policy show --hub-name {0} -n {1}'.format(hub, policy_name), checks=[
            self.check('keyName', policy_name),
            self.check('rights', 'RegistryWrite, ServiceConnect, DeviceConnect')
        ])

        # Test 'az iot hub policy delete'
        self.cmd('iot hub policy delete --hub-name {0} -n {1}'.format(hub, policy_name), checks=self.is_empty())
        self.cmd('iot hub policy list --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 5)
        ])

        # Test 'az iot hub consumer-group create'
        consumer_group_name = 'cg1'
        self.cmd('iot hub consumer-group create --hub-name {0} -n {1}'.format(hub, consumer_group_name), checks=[
            self.check('name', consumer_group_name)
        ])

        # Test 'az iot hub consumer-group show'
        self.cmd('iot hub consumer-group show --hub-name {0} -n {1}'.format(hub, consumer_group_name), checks=[
            self.check('name', consumer_group_name)
        ])

        # Test 'az iot hub consumer-group list'
        self.cmd('iot hub consumer-group list --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 2),
            self.check('[0].name', '$Default'),
            self.check('[1].name', consumer_group_name)
        ])

        # Test 'az iot hub consumer-group delete'
        self.cmd('iot hub consumer-group delete --hub-name {0} -n {1}'.format(hub, consumer_group_name),
                 checks=self.is_empty())

        self.cmd('iot hub consumer-group list --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 1),
            self.check('[0].name', '$Default')
        ])

        # Test 'az iot hub job list'
        self.cmd('iot hub job list --hub-name {0}'.format(hub), checks=self.is_empty())

        # Test 'az iot hub job show'
        job_id = 'fake-job'
        self.cmd('iot hub job show --hub-name {0} --job-id {1}'.format(hub, job_id),
                 expect_failure=True)

        # Test 'az iot hub job cancel'
        self.cmd('iot hub job cancel --hub-name {0} --job-id {1}'.format(hub, job_id),
                 expect_failure=True)

        # Test 'az iot device create'
        device_1 = device_id_1
        self.cmd('iot device create --hub-name {0} -d {1}'.format(hub, device_1),
                 checks=[self.check('deviceId', device_1),
                         self.check('status', 'enabled'),
                         self.check('statusReason', None),
                         self.check('connectionState', 'Disconnected')])

        device_2 = device_id_2
        primary_thumbprint = 'A361EA6A7119A8B0B7BBFFA2EAFDAD1F9D5BED8C'
        secondary_thumbprint = '14963E8F3BA5B3984110B3C1CA8E8B8988599087'
        self.cmd('iot device create --hub-name {0} -d {1} --x509 --primary-thumbprint {2} --secondary-thumbprint {3}'
                 .format(hub, device_2, primary_thumbprint, secondary_thumbprint),
                 checks=[self.check('deviceId', device_2),
                         self.check('status', 'enabled'),
                         self.check('statusReason', None),
                         self.check('connectionState', 'Disconnected'),
                         self.check('authentication.symmetricKey.primaryKey', None),
                         self.check('authentication.symmetricKey.secondaryKey', None),
                         self.check('authentication.x509Thumbprint.primaryThumbprint', primary_thumbprint),
                         self.check('authentication.x509Thumbprint.secondaryThumbprint', secondary_thumbprint)])

        # Test 'az iot device show-connection-string'
        conn_str_pattern = r'^HostName={0}\.azure-devices\.net;DeviceId={1};SharedAccessKey='.format(hub, device_1)
        self.cmd('iot device show-connection-string --hub-name {0} -d {1}'.format(hub, device_1), checks=[
            self.check_pattern('connectionString', conn_str_pattern)
        ])

        connection_string = 'HostName={0}.azure-devices.net;DeviceId={1};x509=true'.format(hub, device_2)
        self.cmd('iot device show-connection-string --hub-name {0} -d {1}'.format(hub, device_2), checks=[
            self.check('connectionString', connection_string)
        ])

        device_id_pattern = r'^test\-device\-\d$'
        self.cmd('iot device show-connection-string --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 2),
            self.check_pattern('[0].deviceId', device_id_pattern),
            self.check_pattern('[1].deviceId', device_id_pattern)
        ])

        # Test 'az iot device show'
        self.cmd('iot device show --hub-name {0} -d {1}'.format(hub, device_1), checks=[
            self.check('deviceId', device_1),
            self.check('status', 'enabled'),
            self.check('statusReason', None),
            self.check('connectionState', 'Disconnected')
        ])

        # Test 'az iot device update'
        status_reason = 'TestReason'
        self.cmd('iot device update --hub-name {0} -d {1} --set statusReason={2}'.format(hub, device_2, status_reason),
                 checks=[
                     self.check('deviceId', device_2),
                     self.check('status', 'enabled'),
                     self.check('statusReason', status_reason),
                     self.check('connectionState', 'Disconnected'),
                     self.check('authentication.symmetricKey.primaryKey', None),
                     self.check('authentication.symmetricKey.secondaryKey', None),
                     self.check('authentication.x509Thumbprint.primaryThumbprint', primary_thumbprint),
                     self.check('authentication.x509Thumbprint.secondaryThumbprint', secondary_thumbprint)])

        # Test 'az iot device list'
        self.cmd('iot device list --hub-name {0}'.format(hub),
                 checks=[self.check('length([*])', 2),
                         self.check_pattern('[0].deviceId', device_id_pattern),
                         self.check_pattern('[1].deviceId', device_id_pattern)])

        # Test 'az iot device message send'
        self.cmd('iot device message send --hub-name {0} -d {1}'.format(hub, device_1), checks=self.is_empty())

        # Test 'az iot device message receive'
        self.cmd('iot device message receive --hub-name {0} -d {1}'.format(hub, device_1), checks=self.is_empty())

        # Test 'az iot device message complete'
        lock_token = '00000000-0000-0000-0000-000000000000'
        self.cmd('iot device message complete --hub-name {0} -d {1} --lock-token {2}'.format(hub, device_1, lock_token),
                 expect_failure=True)

        # Test 'az iot device message reject'
        self.cmd('iot device message reject --hub-name {0} -d {1} --lock-token {2}'.format(hub, device_1, lock_token),
                 expect_failure=True)

        # Test 'az iot device message abandon'
        self.cmd('iot device message abandon --hub-name {0} -d {1} --lock-token {2}'.format(hub, device_1, lock_token),
                 expect_failure=True)

        # Test 'az iot hub show-quota-metrics'
        self.cmd('iot hub show-quota-metrics -n {0}'.format(hub), checks=[
            self.check('length([*])', 2),
            self.check('[0].name', 'TotalMessages'),
            self.check('[0].maxValue', 400000),
            self.check('[1].name', 'TotalDeviceCount'),
            self.check('[1].maxValue', 500000)
        ])

        # Test 'az iot hub show-stats'
        device_count_pattern = r'^\d$'
        self.cmd('iot hub show-stats -n {0}'.format(hub), checks=[
            self.check_pattern('disabledDeviceCount', device_count_pattern),
            self.check_pattern('enabledDeviceCount', device_count_pattern),
            self.check_pattern('totalDeviceCount', device_count_pattern)
        ])

        # Test 'az iot device delete'
        self.cmd('iot device delete --hub-name {0} -d {1}'.format(hub, device_1), checks=self.is_empty())
        self.cmd('iot device delete --hub-name {0} -d {1}'.format(hub, device_2), checks=self.is_empty())

        # Test 'az iot hub delete'
        self.cmd('iot hub delete -n {0}'.format(hub), checks=self.is_empty())



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from datetime import datetime
from dateutil.tz import tzutc   # pylint: disable=import-error

from azure.cli.core.util import CLIError
from azure.cli.testsdk.base import execute
from azure.cli.testsdk.exceptions import CliTestError   # pylint: disable=unused-import
from azure.cli.testsdk import (
    JMESPathCheck,
    NoneCheck,
    ResourceGroupPreparer,
    ScenarioTest)
from azure.cli.testsdk.preparers import (
    AbstractPreparer,
    SingleValueReplacer)


# Constants
SERVER_NAME_PREFIX = 'azuredbclitest'
SERVER_NAME_MAX_LENGTH = 63


class ServerPreparer(AbstractPreparer, SingleValueReplacer):
    # pylint: disable=too-many-instance-attributes
    def __init__(self, engine_type='mysql', engine_parameter_name='database_engine',
                 name_prefix=SERVER_NAME_PREFIX, parameter_name='server', location='koreasouth',
                 admin_user='cloudsa', admin_password='SecretPassword123',
                 resource_group_parameter_name='resource_group', skip_delete=True,
                 sku_name='GP_Gen5_2'):
        super(ServerPreparer, self).__init__(name_prefix, SERVER_NAME_MAX_LENGTH)
        from azure.cli.testsdk import TestCli
        self.cli_ctx = TestCli()
        self.engine_type = engine_type
        self.engine_parameter_name = engine_parameter_name
        self.location = location
        self.parameter_name = parameter_name
        self.admin_user = admin_user
        self.admin_password = admin_password
        self.resource_group_parameter_name = resource_group_parameter_name
        self.skip_delete = skip_delete
        self.sku_name = sku_name

    def create_resource(self, name, **kwargs):
        group = self._get_resource_group(**kwargs)
        template = 'az {} server create -l {} -g {} -n {} -u {} -p {} --sku-name {}'
        execute(self.cli_ctx, template.format(self.engine_type,
                                              self.location,
                                              group, name,
                                              self.admin_user,
                                              self.admin_password,
                                              self.sku_name))
        return {self.parameter_name: name,
                self.engine_parameter_name: self.engine_type}

    def remove_resource(self, name, **kwargs):
        if not self.skip_delete:
            group = self._get_resource_group(**kwargs)
            execute(self.cli_ctx, 'az {} server delete -g {} -n {} --yes'.format(self.engine_type, group, name))

    def _get_resource_group(self, **kwargs):
        return kwargs.get(self.resource_group_parameter_name)


class ServerMgmtScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(parameter_name='resource_group_1')
    @ResourceGroupPreparer(parameter_name='resource_group_2')
    def test_mysql_server_mgmt(self, resource_group_1, resource_group_2):
        self._test_server_mgmt('mysql', resource_group_1, resource_group_2)

    @ResourceGroupPreparer(parameter_name='resource_group_1')
    @ResourceGroupPreparer(parameter_name='resource_group_2')
    def test_postgres_server_mgmt(self, resource_group_1, resource_group_2):
        self._test_server_mgmt('postgres', resource_group_1, resource_group_2)

    def _test_server_mgmt(self, database_engine, resource_group_1, resource_group_2):
        servers = [self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH),
                   self.create_random_name('azuredbclirestore', SERVER_NAME_MAX_LENGTH),
                   self.create_random_name('azuredbcligeorestore', SERVER_NAME_MAX_LENGTH)]
        admin_login = 'cloudsa'
        admin_passwords = ['SecretPassword123', 'SecretPassword456']
        edition = 'GeneralPurpose'
        backupRetention = 10
        geoRedundantBackup = 'Enabled'
        old_cu = 2
        new_cu = 4
        family = 'Gen5'
        skuname = 'GP_{}_{}'.format(family, old_cu)
        newskuname = 'GP_{}_{}'.format(family, new_cu)
        loc = 'koreasouth'

        geoGeoRedundantBackup = 'Disabled'
        geoBackupRetention = 20
        geoloc = 'koreasouth'

        # test create server
        self.cmd('{} server create -g {} --name {} -l {} '
                 '--admin-user {} --admin-password {} '
                 '--sku-name {} --tags key=1 --geo-redundant-backup {} '
                 '--backup-retention {}'
                 .format(database_engine, resource_group_1, servers[0], loc,
                         admin_login, admin_passwords[0], skuname,
                         geoRedundantBackup, backupRetention),
                 checks=[
                     JMESPathCheck('name', servers[0]),
                     JMESPathCheck('resourceGroup', resource_group_1),
                     JMESPathCheck('administratorLogin', admin_login),
                     JMESPathCheck('sslEnforcement', 'Enabled'),
                     JMESPathCheck('tags.key', '1'),
                     JMESPathCheck('sku.capacity', old_cu),
                     JMESPathCheck('sku.tier', edition),
                     JMESPathCheck('storageProfile.backupRetentionDays', backupRetention),
                     JMESPathCheck('storageProfile.geoRedundantBackup', geoRedundantBackup)])

        # test show server
        result = self.cmd('{} server show -g {} --name {}'
                          .format(database_engine, resource_group_1, servers[0]),
                          checks=[
                              JMESPathCheck('name', servers[0]),
                              JMESPathCheck('resourceGroup', resource_group_1),
                              JMESPathCheck('administratorLogin', admin_login),
                              JMESPathCheck('sslEnforcement', 'Enabled'),
                              JMESPathCheck('tags.key', '1'),
                              JMESPathCheck('sku.capacity', old_cu),
                              JMESPathCheck('sku.tier', edition),
                              JMESPathCheck('storageProfile.backupRetentionDays', backupRetention),
                              JMESPathCheck('storageProfile.geoRedundantBackup', geoRedundantBackup)]).get_output_in_json()  # pylint: disable=line-too-long

        # test update server
        self.cmd('{} server update -g {} --name {} --admin-password {} '
                 '--ssl-enforcement Disabled --tags key=2'
                 .format(database_engine, resource_group_1, servers[0], admin_passwords[1]),
                 checks=[
                     JMESPathCheck('name', servers[0]),
                     JMESPathCheck('resourceGroup', resource_group_1),
                     JMESPathCheck('sslEnforcement', 'Disabled'),
                     JMESPathCheck('sku.tier', edition),
                     JMESPathCheck('tags.key', '2'),
                     JMESPathCheck('administratorLogin', admin_login)])

        self.cmd('{} server update -g {} --name {} --sku-name {}'
                 .format(database_engine, resource_group_1, servers[0], newskuname),
                 checks=[
                     JMESPathCheck('name', servers[0]),
                     JMESPathCheck('resourceGroup', resource_group_1),
                     JMESPathCheck('sku.tier', edition),
                     JMESPathCheck('sku.capacity', new_cu),
                     JMESPathCheck('administratorLogin', admin_login)])

        # test show server
        self.cmd('{} server show -g {} --name {}'
                 .format(database_engine, resource_group_1, servers[0]),
                 checks=[
                     JMESPathCheck('name', servers[0]),
                     JMESPathCheck('resourceGroup', resource_group_1),
                     JMESPathCheck('sslEnforcement', 'Disabled'),
                     JMESPathCheck('sku.tier', edition),
                     JMESPathCheck('sku.capacity', new_cu),
                     JMESPathCheck('tags.key', '2'),
                     JMESPathCheck('administratorLogin', admin_login)])

        # test update server per property
        self.cmd('{} server update -g {} --name {} --sku-name {}'
                 .format(database_engine, resource_group_1, servers[0], skuname),
                 checks=[
                     JMESPathCheck('name', servers[0]),
                     JMESPathCheck('resourceGroup', resource_group_1),
                     JMESPathCheck('sku.tier', edition),
                     JMESPathCheck('sku.capacity', old_cu),
                     JMESPathCheck('administratorLogin', admin_login)])

        self.cmd('{} server update -g {} --name {} --ssl-enforcement Enabled'
                 .format(database_engine, resource_group_1, servers[0]),
                 checks=[
                     JMESPathCheck('name', servers[0]),
                     JMESPathCheck('resourceGroup', resource_group_1),
                     JMESPathCheck('sslEnforcement', 'Enabled'),
                     JMESPathCheck('sku.tier', edition),
                     JMESPathCheck('administratorLogin', admin_login)])

        self.cmd('{} server update -g {} --name {} --tags key=3'
                 .format(database_engine, resource_group_1, servers[0]),
                 checks=[
                     JMESPathCheck('name', servers[0]),
                     JMESPathCheck('resourceGroup', resource_group_1),
                     JMESPathCheck('sslEnforcement', 'Enabled'),
                     JMESPathCheck('tags.key', '3'),
                     JMESPathCheck('sku.tier', edition),
                     JMESPathCheck('administratorLogin', admin_login)])

        # test restore to a new server, make sure wait at least 5 min after server created.
        from time import sleep
        sleep(300)

        self.cmd('{} server restore -g {} --name {} '
                 '--source-server {} '
                 '--restore-point-in-time {}'
                 .format(database_engine, resource_group_2, servers[1], result['id'],
                         datetime.utcnow().replace(tzinfo=tzutc()).isoformat()),
                 checks=[
                     JMESPathCheck('name', servers[1]),
                     JMESPathCheck('resourceGroup', resource_group_2),
                     JMESPathCheck('sku.tier', edition),
                     JMESPathCheck('administratorLogin', admin_login)])

        # test georestore server
        with self.assertRaises(CLIError) as exception:
            self.cmd('{} server georestore -g {} --name {} --source-server {} -l {} '
                     '--geo-redundant-backup {} --backup-retention {}'
                     .format(database_engine, resource_group_2, servers[2], result['id'],
                             geoloc, geoGeoRedundantBackup, geoBackupRetention),
                     checks=[
                         JMESPathCheck('name', servers[2]),
                         JMESPathCheck('resourceGroup', resource_group_2),
                         JMESPathCheck('sku.tier', edition),
                         JMESPathCheck('administratorLogin', admin_login),
                         JMESPathCheck('location', geoloc),
                         JMESPathCheck('storageProfile.backupRetentionDays', geoBackupRetention),
                         JMESPathCheck('storageProfile.geoRedundantBackup', geoGeoRedundantBackup)])
        self.assertTrue(' does not have the server ' in '{}'.format(exception.exception))

        # test list servers
        self.cmd('{} server list -g {}'.format(database_engine, resource_group_2),
                 checks=[JMESPathCheck('type(@)', 'array')])

        # test list servers without resource group
        self.cmd('{} server list'.format(database_engine),
                 checks=[JMESPathCheck('type(@)', 'array')])

        # test delete server
        self.cmd('{} server delete -g {} --name {} --yes'
                 .format(database_engine, resource_group_1, servers[0]), checks=NoneCheck())
        self.cmd('{} server delete -g {} -n {} --yes'
                 .format(database_engine, resource_group_2, servers[1]), checks=NoneCheck())

        # test list server should be 0
        self.cmd('{} server list -g {}'.format(database_engine, resource_group_1), checks=[NoneCheck()])
        self.cmd('{} server list -g {}'.format(database_engine, resource_group_2), checks=[NoneCheck()])


class ProxyResourcesMgmtScenarioTest(ScenarioTest):

    @ResourceGroupPreparer()
    @ServerPreparer(engine_type='mysql')
    def test_mysql_proxy_resources_mgmt(self, resource_group, server, database_engine):
        self._test_firewall_mgmt(resource_group, server, database_engine)
        self._test_db_mgmt(resource_group, server, database_engine)
        self._test_configuration_mgmt(resource_group, server, database_engine)
        self._test_log_file_mgmt(resource_group, server, database_engine)

    @ResourceGroupPreparer()
    @ServerPreparer(engine_type='postgres')
    def test_postgres_proxy_resources_mgmt(self, resource_group, server, database_engine):
        self._test_firewall_mgmt(resource_group, server, database_engine)
        self._test_db_mgmt(resource_group, server, database_engine)
        self._test_configuration_mgmt(resource_group, server, database_engine)
        self._test_log_file_mgmt(resource_group, server, database_engine)

    def _test_firewall_mgmt(self, resource_group, server, database_engine):
        firewall_rule_1 = 'rule1'
        start_ip_address_1 = '0.0.0.0'
        end_ip_address_1 = '255.255.255.255'
        firewall_rule_2 = 'rule2'
        start_ip_address_2 = '123.123.123.123'
        end_ip_address_2 = '123.123.123.124'

        # test firewall-rule create
        self.cmd('{} server firewall-rule create -n {} -g {} -s {} '
                 '--start-ip-address {} --end-ip-address {}'
                 .format(database_engine, firewall_rule_1, resource_group, server,
                         start_ip_address_1, end_ip_address_1),
                 checks=[
                     JMESPathCheck('name', firewall_rule_1),
                     JMESPathCheck('resourceGroup', resource_group),
                     JMESPathCheck('startIpAddress', start_ip_address_1),
                     JMESPathCheck('endIpAddress', end_ip_address_1)])

        # test firewall-rule show
        self.cmd('{} server firewall-rule show --name {} -g {} --server {}'
                 .format(database_engine, firewall_rule_1, resource_group, server),
                 checks=[
                     JMESPathCheck('name', firewall_rule_1),
                     JMESPathCheck('resourceGroup', resource_group),
                     JMESPathCheck('startIpAddress', start_ip_address_1),
                     JMESPathCheck('endIpAddress', end_ip_address_1)])

        # test firewall-rule update
        self.cmd('{} server firewall-rule update -n {} -g {} -s {} '
                 '--start-ip-address {} --end-ip-address {}'
                 .format(database_engine, firewall_rule_1, resource_group, server,
                         start_ip_address_2, end_ip_address_2),
                 checks=[
                     JMESPathCheck('name', firewall_rule_1),
                     JMESPathCheck('resourceGroup', resource_group),
                     JMESPathCheck('startIpAddress', start_ip_address_2),
                     JMESPathCheck('endIpAddress', end_ip_address_2)])

        self.cmd('{} server firewall-rule update --name {} -g {} --server {} '
                 '--start-ip-address {}'
                 .format(database_engine, firewall_rule_1, resource_group, server,
                         start_ip_address_1),
                 checks=[
                     JMESPathCheck('name', firewall_rule_1),
                     JMESPathCheck('resourceGroup', resource_group),
                     JMESPathCheck('startIpAddress', start_ip_address_1),
                     JMESPathCheck('endIpAddress', end_ip_address_2)])

        self.cmd('{} server firewall-rule update -n {} -g {} -s {} '
                 '--end-ip-address {}'
                 .format(database_engine, firewall_rule_1, resource_group, server,
                         end_ip_address_1),
                 checks=[
                     JMESPathCheck('name', firewall_rule_1),
                     JMESPathCheck('resourceGroup', resource_group),
                     JMESPathCheck('startIpAddress', start_ip_address_1),
                     JMESPathCheck('endIpAddress', end_ip_address_1)])

        # test firewall-rule create another rule
        self.cmd('{} server firewall-rule create --name {} -g {} --server {} '
                 '--start-ip-address {} --end-ip-address {}'
                 .format(database_engine, firewall_rule_2, resource_group, server,
                         start_ip_address_2, end_ip_address_2),
                 checks=[
                     JMESPathCheck('name', firewall_rule_2),
                     JMESPathCheck('resourceGroup', resource_group),
                     JMESPathCheck('startIpAddress', start_ip_address_2),
                     JMESPathCheck('endIpAddress', end_ip_address_2)])

        # test firewall-rule list
        self.cmd('{} server firewall-rule list -g {} -s {}'
                 .format(database_engine, resource_group, server), checks=[JMESPathCheck('length(@)', 2)])

        self.cmd('{} server firewall-rule delete --name {} -g {} --server {} --yes'
                 .format(database_engine, firewall_rule_1, resource_group, server), checks=NoneCheck())
        self.cmd('{} server firewall-rule list -g {} --server {}'
                 .format(database_engine, resource_group, server), checks=[JMESPathCheck('length(@)', 1)])
        self.cmd('{} server firewall-rule delete -n {} -g {} -s {} --yes'
                 .format(database_engine, firewall_rule_2, resource_group, server), checks=NoneCheck())
        self.cmd('{} server firewall-rule list -g {} --server {}'
                 .format(database_engine, resource_group, server), checks=[NoneCheck()])

    def _test_db_mgmt(self, resource_group, server, database_engine):
        self.cmd('{} db list -g {} -s {}'.format(database_engine, resource_group, server),
                 checks=JMESPathCheck('type(@)', 'array'))

    def _test_configuration_mgmt(self, resource_group, server, database_engine):
        if database_engine == 'mysql':
            config_name = 'log_slow_admin_statements'
            default_value = 'OFF'
            new_value = 'ON'
        else:
            config_name = 'array_nulls'
            default_value = 'on'
            new_value = 'off'

        # test show configuration
        self.cmd('{} server configuration show --name {} -g {} --server {}'
                 .format(database_engine, config_name, resource_group, server),
                 checks=[
                     JMESPathCheck('name', config_name),
                     JMESPathCheck('value', default_value),
                     JMESPathCheck('source', 'system-default')])

        # test update configuration
        self.cmd('{} server configuration set -n {} -g {} -s {} --value {}'
                 .format(database_engine, config_name, resource_group, server, new_value),
                 checks=[
                     JMESPathCheck('name', config_name),
                     JMESPathCheck('value', new_value),
                     JMESPathCheck('source', 'user-override')])

        self.cmd('{} server configuration set -n {} -g {} -s {}'
                 .format(database_engine, config_name, resource_group, server),
                 checks=[
                     JMESPathCheck('name', config_name),
                     JMESPathCheck('value', default_value)])

        # test list configurations
        self.cmd('{} server configuration list -g {} -s {}'
                 .format(database_engine, resource_group, server),
                 checks=[JMESPathCheck('type(@)', 'array')])

    def _test_log_file_mgmt(self, resource_group, server, database_engine):
        if database_engine == 'mysql':
            config_name = 'slow_query_log'
            new_value = 'ON'

            # test update configuration
            self.cmd('{} server configuration set -n {} -g {} -s {} --value {}'
                     .format(database_engine, config_name, resource_group, server, new_value),
                     checks=[
                         JMESPathCheck('name', config_name),
                         JMESPathCheck('value', new_value)])

        # test list log files
        # ensure recording good for at least 5 years!
        result = self.cmd('{} server-logs list -g {} -s {} --file-last-written 43800'
                          .format(database_engine, resource_group, server),
                          checks=[
                              JMESPathCheck('length(@)', 1),
                              JMESPathCheck('type(@)', 'array')]).get_output_in_json()

        self.assertIsNotNone(result[0]['name'])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


import json

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, JMESPathCheck, JMESPathCheckGreaterThan


class DmsServiceTests(ScenarioTest):
    service_random_name_prefix = 'dmsclitest'
    location_name = 'centralus'
    sku_name = 'Basic_2vCores'
    vsubnet_id = '/subscriptions/{}/resourceGroups/ERNetwork/providers/Microsoft.Network/virtualNetworks/AzureDMS-CORP-USC-VNET-5044/subnets/Subnet-1'
    name_exists_checks = [JMESPathCheck('nameAvailable', False),
                          JMESPathCheck('reason', 'AlreadyExists')]
    name_available_checks = [JMESPathCheck('nameAvailable', True)]

    @ResourceGroupPreparer(name_prefix='dms_cli_test', location=location_name)
    def test_service_commands(self, resource_group):
        service_name = self.create_random_name(self.service_random_name_prefix, 15)

        self.kwargs.update({
            'lname': self.location_name,
            'skuname': self.sku_name,
            'vnetid': self.vsubnet_id.format(self.get_subscription_id()),
            'sname': service_name
        })

        skus_checks = [JMESPathCheckGreaterThan('length(@)', 0)]
        self.cmd('az dms list-skus', checks=skus_checks)

        self.cmd('az dms show -g {rg} -n {sname}', expect_failure=True)

        create_checks = [JMESPathCheck('location', self.location_name),
                         JMESPathCheck('name', service_name),
                         JMESPathCheck('resourceGroup', resource_group),
                         JMESPathCheck('sku.name', self.sku_name),
                         JMESPathCheck('virtualSubnetId', self.kwargs['vnetid']),
                         JMESPathCheck('provisioningState', 'Succeeded'),
                         JMESPathCheck('tags.area', 'cli'),
                         JMESPathCheck('tags.env', 'test'),
                         JMESPathCheck('type', 'Microsoft.DataMigration/services')]
        self.cmd('az dms create -l {lname} -n {sname} -g {rg} --sku-name {skuname} --subnet {vnetid} --tags area=cli env=test', checks=create_checks)

        self.cmd('az dms show -g {rg} -n {sname}', checks=create_checks)

        list_checks = [JMESPathCheck('length(@)', 1)]
        self.cmd('az dms list -g {rg}', checks=list_checks)

        status_online_checks = [JMESPathCheck('status', 'Online'),
                                JMESPathCheck('vmSize', 'Standard_A2_v2')]
        self.cmd('az dms check-status -g {rg} -n {sname}', checks=status_online_checks)

        self.cmd('az dms stop -g {rg} -n {sname}')

        status_offline_checks = [JMESPathCheck('status', 'Stopped'),
                                 JMESPathCheck('vmSize', None)]
        self.cmd('az dms check-status -g {rg} -n {sname}', checks=status_offline_checks)

        self.cmd('az dms start -g {rg} -n {sname}')

        self.cmd('az dms check-status -g {rg} -n {sname}', checks=status_online_checks)

        self.cmd('az dms check-name -l {lname} -n {sname}', checks=self.name_exists_checks)

        self.cmd('az dms delete -g {rg} -n {sname} -y')

        self.cmd('az dms check-name -l {lname} -n {sname}', checks=self.name_available_checks)

    @ResourceGroupPreparer(name_prefix='dms_cli_test_', location=location_name)
    def test_project_commands(self, resource_group):
        service_name = self.create_random_name(self.service_random_name_prefix, 15)
        project_name1 = self.create_random_name('project1', 15)
        project_name2 = self.create_random_name('project2', 15)

        self.kwargs.update({
            'lname': self.location_name,
            'skuname': self.sku_name,
            'vnetid': self.vsubnet_id.format(self.get_subscription_id()),
            'sname': service_name,
            'pname1': project_name1,
            'pname2': project_name2
        })

        # Set up container service
        self.cmd('az dms create -l {lname} -n {sname} -g {rg} --sku-name {skuname} --subnet {vnetid}')

        self.cmd('az dms project show -g {rg} --service-name {sname} -n {pname1}', expect_failure=True)

        create_checks = [JMESPathCheck('location', self.location_name),
                         JMESPathCheck('resourceGroup', resource_group),
                         JMESPathCheck('name', project_name1),
                         JMESPathCheck('sourcePlatform', 'SQL'),
                         JMESPathCheck('targetPlatform', 'SQLDB'),
                         JMESPathCheck('provisioningState', 'Succeeded'),
                         JMESPathCheck('tags.Cli', ''),
                         JMESPathCheck('tags.Type', 'test'),
                         JMESPathCheck('type', 'Microsoft.DataMigration/services/projects')]
        self.cmd('az dms project create -g {rg} --service-name {sname} -l {lname} -n {pname1} --source-platform SQL --target-platform SQLDB --tags Type=test Cli', checks=create_checks)

        self.cmd('az dms project show -g {rg} --service-name {sname} -n {pname1}', create_checks)

        create_checks_notags = [JMESPathCheck('tags', None)]
        self.cmd('az dms project create -g {rg} --service-name {sname} -l {lname} -n {pname2} --source-platform SQL --target-platform SQLDB', checks=create_checks_notags)

        list_checks = [JMESPathCheck('length(@)', 2),
                       JMESPathCheck("length([?name == '{}'])".format(project_name1), 1)]
        self.cmd('az dms project list -g {rg} --service-name {sname}', list_checks)

        self.cmd('az dms project check-name -g {rg} --service-name {sname} -n {pname2}', checks=self.name_exists_checks)

        self.cmd('az dms project delete -g {rg} --service-name {sname} -n {pname2} -y')

        self.cmd('az dms project check-name -g {rg} --service-name {sname} -n {pname2}', checks=self.name_available_checks)

        # Clean up service for live runs
        self.cmd('az dms delete -g {rg} -n {sname} --delete-running-tasks true -y')

    @ResourceGroupPreparer(name_prefix='dms_cli_test_', location=location_name)
    def test_task_commands(self, resource_group):
        service_name = self.create_random_name(self.service_random_name_prefix, 15)
        project_name = self.create_random_name('project', 15)
        task_name1 = self.create_random_name('task1', 15)
        task_name2 = self.create_random_name('task2', 15)
        database_options1 = "[ { 'name': 'SourceDatabase1', 'target_database_name': 'TargetDatabase1', 'make_source_db_read_only': False, 'table_map': { 'dbo.TestTableSource1': 'dbo.TestTableTarget1', 'dbo.TestTableSource2': 'dbo.TestTableTarget2' } } ]"
        database_options2 = "[ { 'name': 'SourceDatabase2', 'target_database_name': 'TargetDatabase2', 'make_source_db_read_only': False, 'table_map': { 'dbo.TestTableSource1': 'dbo.TestTableTarget1', 'dbo.TestTableSource2': 'dbo.TestTableTarget2' } } ]"
        source_connection_info = "{ 'userName': 'testuser', 'password': 'testpassword', 'dataSource': 'notarealsourceserver', 'authentication': 'SqlAuthentication', 'encryptConnection': True, 'trustServerCertificate': True }"
        target_connection_info = "{ 'userName': 'testuser', 'password': 'testpassword', 'dataSource': 'notarealtargetserver', 'authentication': 'SqlAuthentication', 'encryptConnection': True, 'trustServerCertificate': True }"

        self.kwargs.update({
            'lname': self.location_name,
            'skuname': self.sku_name,
            'vnetid': self.vsubnet_id.format(self.get_subscription_id()),
            'sname': service_name,
            'pname': project_name,
            'tname1': task_name1,
            'tname2': task_name2,
            'dboptions1': database_options1,
            'dboptions2': database_options2,
            'sconn': source_connection_info,
            'tconn': target_connection_info
        })

        # Set up container service and project
        self.cmd('az dms create -l {lname} -n {sname} -g {rg} --sku-name {skuname} --subnet {vnetid}')
        self.cmd('az dms project create -g {rg} --service-name {sname} -l {lname} -n {pname} --source-platform SQL --target-platform SQLDB')

        self.cmd('az dms project task show -g {rg} --service-name {sname} --project-name {pname} -n {tname1}', expect_failure=True)

        create_checks = [JMESPathCheck('name', task_name1),
                         JMESPathCheck('resourceGroup', resource_group),
                         JMESPathCheck('type', 'Microsoft.DataMigration/services/projects/tasks'),
                         JMESPathCheck('length(properties.input.selectedDatabases[0].tableMap)', 2),
                         JMESPathCheck('properties.input.sourceConnectionInfo.dataSource', 'notarealsourceserver'),
                         JMESPathCheck('properties.input.targetConnectionInfo.dataSource', 'notarealtargetserver'),
                         JMESPathCheck('properties.taskType', 'Migrate.SqlServer.SqlDb'),
                         JMESPathCheck('properties.input.validationOptions.enableDataIntegrityValidation', False),
                         JMESPathCheck('properties.input.validationOptions.enableQueryAnalysisValidation', False),
                         JMESPathCheck('properties.input.validationOptions.enableSchemaValidation', False)]
        self.cmd('az dms project task create --database-options-json "{dboptions1}" -n {tname1} --project-name {pname} -g {rg} --service-name {sname} --source-connection-json "{sconn}" --target-connection-json "{tconn}"', checks=create_checks)

        self.cmd('az dms project task show -g {rg} --service-name {sname} --project-name {pname} -n {tname1}', checks=create_checks)

        cancel_checks = [JMESPathCheck('name', task_name1),
                         JMESPathCheck('properties.state', 'Canceled')]
        self.cmd('az dms project task cancel -g {rg} --service-name {sname} --project-name {pname} -n {tname1}', checks=cancel_checks)

        self.cmd('az dms project task create --database-options-json "{dboptions2}" -n {tname2} --project-name {pname} -g {rg} --service-name {sname} --source-connection-json "{sconn}" --target-connection-json "{tconn}"')

        list_checks = [JMESPathCheck('length(@)', 2),
                       JMESPathCheck("length([?name == '{}'])".format(task_name1), 1)]
        self.cmd('az dms project task list -g {rg} --service-name {sname} --project-name {pname} --task-type "Migrate.SqlServer.SqlDb"', checks=list_checks)

        self.cmd('az dms project task check-name -g {rg} --service-name {sname} --project-name {pname} -n {tname1}', checks=self.name_exists_checks)

        self.cmd('az dms project task delete -g {rg} --service-name {sname} --project-name {pname} -n {tname1} --delete-running-tasks true -y')

        self.cmd('az dms project task check-name -g {rg} --service-name {sname} --project-name {pname} -n {tname1}', checks=self.name_available_checks)

        # Clean up service for live runs
        self.cmd('az dms delete -g {rg} -n {sname} --delete-running-tasks true -y')



# This Python file uses the following encoding: utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
from azure.cli.testsdk import LiveScenarioTest, ResourceGroupPreparer, StorageAccountPreparer, JMESPathCheck
from .storage_test_util import StorageScenarioMixin


class StorageCopyTests(StorageScenarioMixin, LiveScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_blob_copy_non_ascii(self, storage_account_info):
        src_container = self.create_container(storage_account_info)
        dst_container = self.create_container(storage_account_info)

        source_file = self.create_temp_file(16, full_random=False)
        blobs = ['blobğşŞ', 'blobÉ®']

        for blob_name in blobs:
            self.storage_cmd('storage blob upload -c {} -f "{}" -n {}', storage_account_info,
                             src_container, source_file, blob_name)
            self.storage_cmd('storage blob exists -c {} -n {}', storage_account_info,
                             src_container, blob_name).assert_with_checks(JMESPathCheck('exists', True))
            url = self.storage_cmd('storage blob url -n {} -c {}', storage_account_info,
                                   blob_name, src_container).get_output_in_json()
            self.storage_cmd('storage blob copy start -c {} -b {} -u {}',
                             storage_account_info, dst_container, blob_name, url).assert_with_checks([
                                 JMESPathCheck('status', "success")])
            self.storage_cmd('storage blob exists -c {} -n {}', storage_account_info,
                             dst_container, blob_name).assert_with_checks(JMESPathCheck('exists', True))

        self.storage_cmd('storage blob copy start-batch --destination-container {} --pattern * --source-container {}',
                         storage_account_info, dst_container, src_container).assert_with_checks(
                             JMESPathCheck('length(@)', 2))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_file_copy_non_ascii(self, storage_account_info):
        src_share = self.create_share(storage_account_info)
        dst_share = self.create_share(storage_account_info)

        source_dir = self.create_temp_dir()
        files = ['fileğşŞ', 'file的Φ']
        for file_name in files:
            with open(os.path.join(source_dir, file_name), 'wb') as temp_file:
                temp_file.write(bytearray([0] * 1024))
            self.storage_cmd('storage file upload -s {} --source "{}"', storage_account_info,
                             src_share, os.path.join(source_dir, file_name))
            self.storage_cmd('storage file exists -s {} -p "{}"', storage_account_info,
                             src_share, file_name).assert_with_checks(JMESPathCheck('exists', True))
            url = self.storage_cmd('storage file url -p "{}" -s {}', storage_account_info,
                                   file_name, src_share).get_output_in_json()
            self.storage_cmd('storage file copy start -s {} -p "{}" -u {}',
                             storage_account_info, dst_share, file_name, url).assert_with_checks([
                                 JMESPathCheck('status', "success")])
            self.storage_cmd('storage file exists -s {} -p "{}"', storage_account_info,
                             dst_share, file_name).assert_with_checks(JMESPathCheck('exists', True))

        self.storage_cmd('storage file copy start-batch --destination-share {} --pattern * --source-share {}',
                         storage_account_info, dst_share, src_share).assert_with_checks(
                             JMESPathCheck('length(@)', 2))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
import re

from knack.util import CLIError

from azure.cli.command_modules.monitor.operations.activity_log import (_build_activity_log_odata_filter,
                                                                       _activity_log_select_filter_builder,
                                                                       _build_odata_filter)


class TestActivityLogODataBuilderComponents(unittest.TestCase):
    def test_build_activity_logs_odata_filter(self):
        correlation_id = '1234-34567-56789-34567'
        resource_group = 'test'
        resource_id = '/subscriptions/xxxx-xxxx-xxxx-xxx/resourceGroups/testrg/providers/Microsoft.Web/sites/testwebapp'
        resource_provider = 'Microsoft.Web'
        caller = 'contoso@contoso.com'
        status = 'RunsSucceeded'

        filter_output = _build_activity_log_odata_filter(correlation_id)
        regex = r'^(eventTimestamp ge).*(eventTimestamp le).*(correlationId eq).*$'
        assert bool(re.search(regex, filter_output))

        filter_output = _build_activity_log_odata_filter(resource_group=resource_group)
        regex = r'^(eventTimestamp ge).*(eventTimestamp le).*(resourceGroupName eq).*$'
        assert bool(re.search(regex, filter_output))

        filter_output = _build_activity_log_odata_filter(resource_id=resource_id)
        regex = r'^(eventTimestamp ge).*(eventTimestamp le).*(resourceId eq).*$'
        assert bool(re.search(regex, filter_output))

        filter_output = _build_activity_log_odata_filter(resource_provider=resource_provider)
        regex = r'^(eventTimestamp ge).*(eventTimestamp le).*(resourceProvider eq).*$'
        assert bool(re.search(regex, filter_output))

        filter_output = _build_activity_log_odata_filter(caller=caller)
        regex = r'^(eventTimestamp ge).*(eventTimestamp le).*(caller eq).*$'
        assert bool(re.search(regex, filter_output))

        filter_output = _build_activity_log_odata_filter(status=status)
        regex = r'^(eventTimestamp ge).*(eventTimestamp le).*(status eq).*$'
        assert bool(re.search(regex, filter_output))

    def test_activity_logs_select_filter_builder(self):
        select_output = _activity_log_select_filter_builder()
        assert select_output is None

        events = ['channels']
        select_output = _activity_log_select_filter_builder(events)
        assert select_output == '{}'.format(events[0])

        events = ['eventDataId', 'eventSource']
        select_output = _activity_log_select_filter_builder(events)
        assert select_output == '{} , {}'.format(events[0], events[1])

    def test_build_odata_filter(self):
        default_filter = "timeGrain eq duration'PT1M'"
        field_name = 'correlation_id'
        field_value = '1234-34567-56789-34567'
        field_label = 'correlationId'

        filter_output = _build_odata_filter(default_filter, field_name, field_value, field_label)
        regex = r'^({} and {} eq \'{}\')$'.format(default_filter, field_label, field_value)
        assert bool(re.search(regex, filter_output))

        with self.assertRaises(CLIError):
            _build_odata_filter(default_filter, field_name, None, field_label)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import print_function

import unittest
from collections import OrderedDict
from six import StringIO

from azure.cli.core._output import OutputProducer

from knack.output import format_json, format_table, format_tsv
from knack.util import CommandResultItem, normalize_newlines


class TestCoreCLIOutput(unittest.TestCase):
    def setUp(self):
        self.io = StringIO()

    def tearDown(self):
        self.io.close()

    def test_out_json_valid(self):
        # The JSON output when the input is a dict should be the dict serialized to JSON
        output_producer = OutputProducer(formatter=format_json, file=self.io)
        output_producer.out(CommandResultItem({'active': True, 'id': '0b1f6472'}))
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """{
  "active": true,
  "id": "0b1f6472"
}
"""))

    def test_out_json_from_ordered_dict(self):
        # The JSON output when the input is OrderedDict should be serialized to JSON
        output_producer = OutputProducer(formatter=format_json, file=self.io)
        output_producer.out(CommandResultItem(OrderedDict({'active': True, 'id': '0b1f6472'})))
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """{
  "active": true,
  "id": "0b1f6472"
}
"""))

    def test_out_json_byte(self):
        output_producer = OutputProducer(formatter=format_json, file=self.io)
        output_producer.out(CommandResultItem({'active': True, 'contents': b'0b1f6472'}))
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """{
  "active": true,
  "contents": "0b1f6472"
}
"""))

    def test_out_json_byte_empty(self):
        output_producer = OutputProducer(formatter=format_json, file=self.io)
        output_producer.out(CommandResultItem({'active': True, 'contents': b''}))
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """{
  "active": true,
  "contents": ""
}
"""))

    # TABLE output tests

    def test_out_table(self):
        output_producer = OutputProducer(formatter=format_table, file=self.io)
        obj = OrderedDict()
        obj['active'] = True
        obj['val'] = '0b1f6472'
        output_producer.out(CommandResultItem(obj))
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """Active    Val
--------  --------
True      0b1f6472
"""))

    def test_out_table_list_of_lists(self):
        output_producer = OutputProducer(formatter=format_table, file=self.io)
        obj = [['a', 'b'], ['c', 'd']]
        output_producer.out(CommandResultItem(obj))
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """Column1    Column2
---------  ---------
a          b
c          d
"""))

    def test_out_table_complex_obj(self):
        output_producer = OutputProducer(formatter=format_table, file=self.io)
        obj = OrderedDict()
        obj['name'] = 'qwerty'
        obj['val'] = '0b1f6472qwerty'
        obj['sub'] = {'1'}
        result_item = CommandResultItem(obj)
        output_producer.out(result_item)
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """Name    Val
------  --------------
qwerty  0b1f6472qwerty
"""))

    def test_out_table_no_query_no_transformer_order(self):
        output_producer = OutputProducer(formatter=format_table, file=self.io)
        obj = {'name': 'qwerty', 'val': '0b1f6472qwerty', 'active': True, 'sub': '0b1f6472'}
        result_item = CommandResultItem(obj, table_transformer=None, is_query_active=False)
        output_producer.out(result_item)
        # Should be alphabetical order as no table transformer and query is not active.
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """Active    Name    Sub       Val
--------  ------  --------  --------------
True      qwerty  0b1f6472  0b1f6472qwerty
"""))

    def test_out_table_no_query_yes_transformer_order(self):
        output_producer = OutputProducer(formatter=format_table, file=self.io)
        obj = {'name': 'qwerty', 'val': '0b1f6472qwerty', 'active': True, 'sub': '0b1f6472'}

        def transformer(r):
            return OrderedDict([('Name', r['name']), ('Val', r['val']),
                                ('Active', r['active']), ('Sub', r['sub'])])

        result_item = CommandResultItem(obj, table_transformer=transformer, is_query_active=False)
        output_producer.out(result_item)
        # Should be table transformer order
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """Name    Val             Active    Sub
------  --------------  --------  --------
qwerty  0b1f6472qwerty  True      0b1f6472
"""))

    def test_out_table_no_query_yes_jmespath_table_transformer(self):
        output_producer = OutputProducer(formatter=format_table, file=self.io)
        obj = {'name': 'qwerty', 'val': '0b1f6472qwerty', 'active': True, 'sub': '0b1f6472'}

        result_item = CommandResultItem(obj, table_transformer='{Name:name, Val:val, Active:active}', is_query_active=False)
        output_producer.out(result_item)
        # Should be table transformer order
        self.assertEqual(normalize_newlines(self.io.getvalue()), normalize_newlines(
            """Name    Val             Active
------  --------------  --------
qwerty  0b1f6472qwerty  True
"""))

    # TSV output tests
    def test_output_format_dict(self):
        obj = {}
        obj['A'] = 1
        obj['B'] = 2
        result = format_tsv(CommandResultItem(obj))
        self.assertEqual(result, '1\t2\n')

    def test_output_format_dict_sort(self):
        obj = {}
        obj['B'] = 1
        obj['A'] = 2
        result = format_tsv(CommandResultItem(obj))
        self.assertEqual(result, '2\t1\n')

    def test_output_format_ordereddict_not_sorted(self):
        obj = OrderedDict()
        obj['B'] = 1
        obj['A'] = 2
        result = format_tsv(CommandResultItem(obj))
        self.assertEqual(result, '1\t2\n')

    def test_output_format_ordereddict_list_not_sorted(self):
        obj1 = OrderedDict()
        obj1['B'] = 1
        obj1['A'] = 2

        obj2 = OrderedDict()
        obj2['A'] = 3
        obj2['B'] = 4
        result = format_tsv(CommandResultItem([obj1, obj2]))
        self.assertEqual(result, '1\t2\n3\t4\n')


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer


class TestLogProfileScenarios(ScenarioTest):
    @ResourceGroupPreparer(location='southcentralus')
    @StorageAccountPreparer(location='southcentralus')
    def test_monitor_create_log_profile(self, resource_group, storage_account):
        self.kwargs['name'] = self.create_random_name('clitest', 20)
        self.kwargs['storage'] = self.cmd('storage account show -n {sa} -g {rg} --query id -otsv').output.strip()

        cmd = "monitor log-profiles create --categories 'Write' --enabled false --days 1095 --location southcentralus" \
              " --locations westus southcentralus --name {name} --storage-account-id {storage} -ojson"

        created = self.cmd(cmd).get_output_in_json()

        self.cmd("monitor log-profiles delete -n {name}")

        self.assertEqual(created['storageAccountId'], self.kwargs['storage'])
        self.assertIsNone(created['serviceBusRuleId'])
        self.assertFalse(created['retentionPolicy']['enabled'])



#!/usr/bin/env python

from setuptools import setup

VERSION = "1.0.0.dev13034"

CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'License :: OSI Approved :: MIT License',
]

DEPENDENCIES = [
    'azure-cli',
    'azure-cli-testsdk'
]

setup(
    name='azure-cli-fulltest',
    version=VERSION,
    description='Microsoft Azure Command-Line Tools Full Tests',
    license='MIT',
    author='Microsoft Corporation',
    author_email='azpycli@microsoft.com',
    url='https://github.com/Azure/azure-cli',
    zip_safe=False,
    classifiers=CLASSIFIERS,
    packages=[
        'azure.cli.core.tests',
        'azure.cli.command_modules.acr.tests',
        'azure.cli.command_modules.acr.tests.latest',
        'azure.cli.command_modules.acs.tests',
        'azure.cli.command_modules.acs.tests.latest',
        'azure.cli.command_modules.advisor.tests',
        'azure.cli.command_modules.advisor.tests.latest',
        'azure.cli.command_modules.ams.tests',
        'azure.cli.command_modules.ams.tests.latest',
        'azure.cli.command_modules.appservice.tests',
        'azure.cli.command_modules.appservice.tests.latest',
        'azure.cli.command_modules.backup.tests',
        'azure.cli.command_modules.backup.tests.latest',
        'azure.cli.command_modules.batch.tests',
        'azure.cli.command_modules.batch.tests.latest',
        'azure.cli.command_modules.batchai.tests',
        'azure.cli.command_modules.batchai.tests.latest',
        'azure.cli.command_modules.billing.tests',
        'azure.cli.command_modules.billing.tests.latest',
        'azure.cli.command_modules.cdn.tests',
        'azure.cli.command_modules.cdn.tests.latest',
        'azure.cli.command_modules.cognitiveservices.tests',
        'azure.cli.command_modules.cognitiveservices.tests.latest',
        'azure.cli.command_modules.consumption.tests',
        'azure.cli.command_modules.consumption.tests.latest',
        'azure.cli.command_modules.container.tests',
        'azure.cli.command_modules.container.tests.latest',
        'azure.cli.command_modules.cosmosdb.tests',
        'azure.cli.command_modules.cosmosdb.tests.latest',
        'azure.cli.command_modules.dla.tests',
        'azure.cli.command_modules.dla.tests.latest',
        'azure.cli.command_modules.dls.tests',
        'azure.cli.command_modules.dls.tests.latest',
        'azure.cli.command_modules.dms.tests',
        'azure.cli.command_modules.dms.tests.latest',
        'azure.cli.command_modules.eventgrid.tests',
        'azure.cli.command_modules.eventgrid.tests.latest',
        'azure.cli.command_modules.eventhubs.tests',
        'azure.cli.command_modules.eventhubs.tests.latest',
        'azure.cli.command_modules.extension.tests',
        'azure.cli.command_modules.extension.tests.latest',
        'azure.cli.command_modules.find.tests',
        'azure.cli.command_modules.find.tests.latest',
        'azure.cli.command_modules.interactive.tests',
        'azure.cli.command_modules.interactive.tests.latest',
        'azure.cli.command_modules.iot.tests',
        'azure.cli.command_modules.iot.tests.latest',
        'azure.cli.command_modules.keyvault.tests',
        'azure.cli.command_modules.keyvault.tests.latest',
        'azure.cli.command_modules.lab.tests',
        'azure.cli.command_modules.lab.tests.latest',
        'azure.cli.command_modules.maps.tests',
        'azure.cli.command_modules.maps.tests.latest',
        'azure.cli.command_modules.monitor.tests',
        'azure.cli.command_modules.monitor.tests.latest',
        'azure.cli.command_modules.network.tests',
        'azure.cli.command_modules.network.tests.latest',
        'azure.cli.command_modules.policyinsights.tests',
        'azure.cli.command_modules.policyinsights.tests.latest',
        'azure.cli.command_modules.profile.tests',
        'azure.cli.command_modules.profile.tests.latest',
        'azure.cli.command_modules.rdbms.tests',
        'azure.cli.command_modules.rdbms.tests.latest',
        'azure.cli.command_modules.redis.tests',
        'azure.cli.command_modules.redis.tests.latest',
        'azure.cli.command_modules.reservations.tests',
        'azure.cli.command_modules.reservations.tests.latest',
        'azure.cli.command_modules.resource.tests',
        'azure.cli.command_modules.resource.tests.latest',
        'azure.cli.command_modules.role.tests',
        'azure.cli.command_modules.role.tests.latest',
        'azure.cli.command_modules.servicebus.tests',
        'azure.cli.command_modules.servicebus.tests.latest',
        'azure.cli.command_modules.servicefabric.tests',
        'azure.cli.command_modules.servicefabric.tests.latest',
        'azure.cli.command_modules.sql.tests',
        'azure.cli.command_modules.sql.tests.latest',
        'azure.cli.command_modules.storage.tests',
        'azure.cli.command_modules.storage.tests.latest',
        'azure.cli.command_modules.vm.tests',
        'azure.cli.command_modules.vm.tests.latest',
    ],
    package_data={'': ['recordings/*.yaml',
                       'data/*.zip',
                       'data/*.whl',
                       '*.zip',
                       '*.pem',
                       '*.pfx',
                       '*.txt',
                       '*.json',
                       '*.byok',
                       '*.js',
                       '*.md',
                       '*.bat',
                       '*.txt',
                       '*.cer',
                       '*.yml',
                       '**/*.cer',
                       '**/*.pem',
                       '**/*.pfx',
                       '**/*.txt',
                       '**/*.json',
                       '**/*.byok',
                       '**/*.js',
                       '**/*.md',
                       '**/*.bat',
                       '**/*.txt']},
    install_requires=DEPENDENCIES
)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


import os
import unittest
import mock

from azure.cli.testsdk import TestCli
from azure.cli.command_modules.interactive.azclishell.configuration import Configuration
from azure.cli.command_modules.interactive.azclishell.app import AzInteractiveShell


TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))


class CommandTreeTest(unittest.TestCase):
    """ tests the completion generator """

    def init_tree(self):
        with mock.patch.object(Configuration, 'get_help_files', lambda _: 'help_dump_test.json'):
            with mock.patch.object(Configuration, 'get_config_dir', lambda _: TEST_DIR):
                shell_ctx = AzInteractiveShell(TestCli(), None)
                self.command_tree = shell_ctx.completer.command_tree

    def test_in_tree(self):
        self.init_tree()

        self.assertTrue(self.command_tree.in_tree(['storage']))
        self.assertTrue(self.command_tree.in_tree(['storage', 'account']))
        self.assertTrue(self.command_tree.in_tree(['storage', 'account', 'create']))
        self.assertTrue(self.command_tree.in_tree(['storage', 'account', 'check-name']))

        self.assertTrue(self.command_tree.in_tree(['vm']))
        self.assertTrue(self.command_tree.in_tree(['vmss']))
        self.assertTrue(self.command_tree.in_tree(['vm', 'create']))
        self.assertTrue(self.command_tree.in_tree(['vmss', 'create']))

        self.assertFalse(self.command_tree.in_tree(['']))
        self.assertFalse(self.command_tree.in_tree(['vms']))
        self.assertFalse(self.command_tree.in_tree(['create']))
        self.assertFalse(self.command_tree.in_tree(['vm', 'blah']))
        self.assertFalse(self.command_tree.in_tree(['vm', 'create', 'blah']))

    def test_sub_tree(self):
        self.init_tree()

        tree, current_command, leftover_args = self.command_tree.get_sub_tree([])
        self.assertEqual(tree, self.command_tree)
        self.assertEqual(current_command, '')
        self.assertEqual(leftover_args, [])

        tree, current_command, leftover_args = tree.get_sub_tree(['storage', 'account'])
        self.assertEqual(tree.data, 'account')
        self.assertEqual(current_command, 'storage account')
        self.assertIn('create', tree.children)
        self.assertIn('check-name', tree.children)
        self.assertEqual(leftover_args, [])

        tree, current_command, leftover_args = tree.get_sub_tree(['create'])
        self.assertEqual(tree.data, 'create')
        self.assertFalse(tree.children)
        self.assertEqual(current_command, 'create')
        self.assertEqual(leftover_args, [])

        tree, current_command, leftover_args = self.command_tree.get_sub_tree(
            ['storage', 'account', 'create', '--name', 'MyStorageAccount'])
        self.assertEqual(tree.data, 'create')
        self.assertEqual(current_command, 'storage account create')
        self.assertEqual(leftover_args, ['--name', 'MyStorageAccount'])


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
import mock

from msrestazure.azure_exceptions import CloudError
from azure.mgmt.web.models import (SourceControl, HostNameBinding, Site, SiteConfig,
                                   HostNameSslState, SslState, Certificate,
                                   AddressResponse, HostingEnvironmentProfile)
from azure.mgmt.web import WebSiteManagementClient
from azure.cli.core.adal_authentication import AdalAuthentication
from knack.util import CLIError
from azure.cli.command_modules.appservice.custom import (set_deployment_user,
                                                         update_git_token, add_hostname,
                                                         update_site_configs,
                                                         get_external_ip,
                                                         view_in_browser,
                                                         sync_site_repo,
                                                         _match_host_names_from_cert,
                                                         bind_ssl_cert,
                                                         list_publish_profiles,
                                                         config_source_control,
                                                         show_webapp,
                                                         get_streaming_log,
                                                         download_historical_logs,
                                                         validate_linux_create_options)

# pylint: disable=line-too-long
from vsts_cd_manager.continuous_delivery_manager import ContinuousDeliveryResult


class TestWebappMocked(unittest.TestCase):
    def setUp(self):
        self.client = WebSiteManagementClient(AdalAuthentication(lambda: ('bearer', 'secretToken')), '123455678')

    @mock.patch('azure.cli.command_modules.appservice.custom.web_client_factory', autospec=True)
    def test_set_deployment_user_creds(self, client_factory_mock):
        class MockClient:
            def update_publishing_user(self, user):
                # Don't do an actual call, just return the incoming user
                return user

        client_factory_mock.return_value = MockClient()

        # action
        user = set_deployment_user(mock.MagicMock(), 'admin', 'verySecret1')

        # assert things get wired up with a result returned
        assert user.publishing_user_name == 'admin'
        assert user.publishing_password == 'verySecret1'

    @mock.patch('azure.cli.command_modules.appservice.custom.web_client_factory', autospec=True)
    def test_set_source_control_token(self, client_factory_mock):
        client_factory_mock.return_value = self.client
        self.client._client = mock.MagicMock()
        sc = SourceControl('not-really-needed', source_control_name='GitHub', token='veryNiceToken')
        self.client._client.send.return_value = FakedResponse(200)
        self.client._deserialize = mock.MagicMock()
        self.client._deserialize.return_value = sc

        # action
        result = update_git_token(mock.MagicMock(), 'veryNiceToken')

        # assert things gets wired up
        self.assertEqual(result.token, 'veryNiceToken')

    @mock.patch('azure.cli.command_modules.appservice.custom.web_client_factory', autospec=True)
    def test_set_domain_name(self, client_factory_mock):
        client_factory_mock.return_value = self.client
        # set up the return value for getting a webapp
        webapp = Site('westus')
        webapp.name = 'veryNiceWebApp'
        self.client.web_apps.get = lambda _, _1: webapp

        # set up the result value of putting a domain name
        domain = 'veryNiceDomain'
        binding = HostNameBinding(webapp.location,
                                  domain_id=domain,
                                  custom_host_name_dns_record_type='A',
                                  host_name_type='Managed')
        self.client.web_apps._client = mock.MagicMock()
        self.client.web_apps._client.send.return_value = FakedResponse(200)
        self.client.web_apps._deserialize = mock.MagicMock()
        self.client.web_apps._deserialize.return_value = binding
        # action
        result = add_hostname(mock.MagicMock(), 'g1', webapp.name, domain)

        # assert
        self.assertEqual(result.domain_id, domain)

    @mock.patch('azure.cli.command_modules.appservice.custom.web_client_factory', autospec=True)
    def test_get_external_ip_from_ase(self, client_factory_mock):
        client = mock.Mock()
        client_factory_mock.return_value = client
        cmd_mock = mock.MagicMock()
        # set up the web inside a ASE, with an ip based ssl binding
        host_env = HostingEnvironmentProfile('id11')
        host_env.name = 'ase1'
        host_env.resource_group = 'myRg'

        host_ssl_state = HostNameSslState(ssl_state=SslState.ip_based_enabled, virtual_ip='1.2.3.4')
        client.web_apps.get.return_value = Site('antarctica', hosting_environment_profile=host_env,
                                                host_name_ssl_states=[host_ssl_state])
        client.app_service_environments.list_vips.return_value = AddressResponse()

        # action
        result = get_external_ip(cmd_mock, 'myRg', 'myWeb')

        # assert, we return the virtual ip from the ip based ssl binding
        self.assertEqual('1.2.3.4', result['ip'])

        # tweak to have no ip based ssl binding, but it is in an internal load balancer
        host_ssl_state2 = HostNameSslState(ssl_state=SslState.sni_enabled)
        client.web_apps.get.return_value = Site('antarctica', hosting_environment_profile=host_env,
                                                host_name_ssl_states=[host_ssl_state2])
        client.app_service_environments.list_vips.return_value = AddressResponse(internal_ip_address='4.3.2.1')

        # action
        result = get_external_ip(cmd_mock, 'myRg', 'myWeb')

        # assert, we take the ILB address
        self.assertEqual('4.3.2.1', result['ip'])

        # tweak to have no ip based ssl binding, and not in internal load balancer
        host_ssl_state2 = HostNameSslState(ssl_state=SslState.sni_enabled)
        client.web_apps.get.return_value = Site('antarctica', hosting_environment_profile=host_env,
                                                host_name_ssl_states=[host_ssl_state2])
        client.app_service_environments.list_vips.return_value = AddressResponse(service_ip_address='1.1.1.1')

        # action
        result = get_external_ip(cmd_mock, 'myRg', 'myWeb')

        # assert, we take service ip
        self.assertEqual('1.1.1.1', result['ip'])

    @mock.patch('azure.cli.command_modules.appservice.custom.web_client_factory', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.custom._resolve_hostname_through_dns', autospec=True)
    def test_get_external_ip_from_dns(self, resolve_hostname_mock, client_factory_mock):
        client = mock.Mock()
        client_factory_mock.return_value = client

        # set up the web inside a ASE, with an ip based ssl binding
        site = Site('antarctica')
        site.default_host_name = 'myweb.com'
        client.web_apps.get.return_value = site

        # action
        get_external_ip(mock.MagicMock(), 'myRg', 'myWeb')

        # assert, we return the virtual ip from the ip based ssl binding
        resolve_hostname_mock.assert_called_with('myweb.com')

    @mock.patch('azure.cli.command_modules.appservice.custom.web_client_factory', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.vsts_cd_provider.ContinuousDeliveryManager', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.vsts_cd_provider.Profile', autospec=True)
    def test_config_source_control_vsts(self, profile_mock, cd_manager_mock, client_factory_mock):
        # Mock the result of get auth token (avoiding REST call)
        profile = mock.Mock()
        profile.get_subscription.return_value = {'id': 'id1', 'name': 'sub1', 'tenantId': 'tenant1'}
        profile.get_current_account_user.return_value = None
        profile.get_login_credentials.return_value = None, None, None
        profile.get_access_token_for_resource.return_value = None
        profile_mock.return_value = profile

        # Mock the cd manager class so no REST calls are made
        cd_manager = mock.Mock()
        status = ContinuousDeliveryResult(None, None, None, None, None, None, "message1", None,
                                          None, None)
        cd_manager.setup_continuous_delivery.return_value = status
        cd_manager_mock.return_value = cd_manager

        # Mock the client and set the location
        client = mock.Mock()
        client_factory_mock.return_value = client
        site = Site('antarctica')
        site.default_host_name = 'myweb.com'
        client.web_apps.get.return_value = site

        config_source_control(mock.MagicMock(), 'group1', 'myweb', 'http://github.com/repo1', None, None, None,
                              None, None, 'ASPNet', 'working_directory', 'Gulp', 'Django',
                              'Python 2.7.12 x64', True, 'https://account1.visualstudio.com',
                              None, 'slot1', None, None)
        cd_app_type_details = {
            'cd_app_type': 'ASPNet',
            'app_working_dir': 'working_directory',
            'nodejs_task_runner': 'Gulp',
            'python_framework': 'Django',
            'python_version': 'Python 2.7.12 x64'
        }
        cd_manager.setup_continuous_delivery.assert_called_with('slot1', cd_app_type_details,
                                                                'https://account1.visualstudio.com',
                                                                True, None, None, None)

    @mock.patch('azure.cli.command_modules.appservice.custom._generic_site_operation', autospec=True)
    def test_update_site_config(self, site_op_mock):
        site_config = SiteConfig('antarctica')
        site_op_mock.side_effect = [site_config, None]
        cmd = mock.MagicMock()
        # action
        update_site_configs(cmd, 'myRG', 'myweb', java_version='1.8')
        # assert
        config_for_set = site_op_mock.call_args_list[1][0][5]
        self.assertEqual(config_for_set.java_version, '1.8')
        # point check some unrelated properties should stay at None
        self.assertEqual(config_for_set.use32_bit_worker_process, None)
        self.assertEqual(config_for_set.java_container, None)

    @mock.patch('azure.cli.command_modules.appservice.custom._generic_site_operation', autospec=True)
    def test_list_publish_profiles_on_slots(self, site_op_mock):
        site_op_mock.return_value = [b'<publishData><publishProfile publishUrl="ftp://123"/><publishProfile publishUrl="ftp://1234"/></publishData>']
        # action
        result = list_publish_profiles(mock.MagicMock(), 'myRG', 'myweb', 'slot1')
        # assert
        site_op_mock.assert_called_with(mock.ANY, 'myRG', 'myweb', 'list_publishing_profile_xml_with_secrets', 'slot1')
        self.assertTrue(result[0]['publishUrl'].startswith('ftp://123'))

    @mock.patch('azure.cli.command_modules.appservice.custom._generic_site_operation', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.custom.get_streaming_log', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.custom.open_page_in_browser', autospec=True)
    def test_browse_with_trace(self, webbrowser_mock, log_mock, site_op_mock):
        site = Site('antarctica')
        site.default_host_name = 'haha.com'
        site.enabled_host_names = [site.default_host_name]
        site.host_name_ssl_states = [HostNameSslState('does not matter',
                                                      ssl_state=SslState.ip_based_enabled)]

        site_op_mock.return_value = site
        # action
        view_in_browser(mock.MagicMock(), 'myRG', 'myweb', logs=True)
        # assert
        webbrowser_mock.assert_called_with('https://haha.com')
        log_mock.assert_called_with(mock.ANY, 'myRG', 'myweb', None, None)

    @mock.patch('azure.cli.command_modules.appservice.custom._generic_site_operation', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.custom._rename_server_farm_props', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.custom._fill_ftp_publishing_url', autospec=True)
    def test_show_webapp(self, file_ftp_mock, rename_mock, site_op_mock):
        faked_web = mock.MagicMock()
        site_op_mock.return_value = faked_web
        # action
        result = show_webapp(mock.MagicMock(), 'myRG', 'myweb', slot=None, app_instance=None)
        # assert (we invoke the site op)
        self.assertEqual(faked_web, result)
        self.assertTrue(rename_mock.called)
        self.assertTrue(file_ftp_mock.called)

    @mock.patch('azure.cli.command_modules.appservice.custom._generic_site_operation', autospec=True)
    def test_sync_repository_skip_bad_error(self, site_op_mock):
        resp = FakedResponse(200)  # because of bad spec, sdk throws on 200.
        setattr(resp, 'text', '{"Message": ""}')
        site_op_mock.side_effect = CloudError(resp, error="bad error")
        # action
        sync_site_repo(mock.MagicMock(), 'myRG', 'myweb')
        # assert
        pass  # if we are here, it means CLI has captured the bogus exception

    def test_match_host_names_from_cert(self):
        result = _match_host_names_from_cert(['*.mysite.com'], ['admin.mysite.com', 'log.mysite.com', 'mysite.com'])
        self.assertEqual(set(['admin.mysite.com', 'log.mysite.com']), result)

        result = _match_host_names_from_cert(['*.mysite.com', 'mysite.com'], ['admin.mysite.com', 'log.mysite.com', 'mysite.com'])
        self.assertEqual(set(['admin.mysite.com', 'log.mysite.com', 'mysite.com']), result)

    @mock.patch('azure.cli.command_modules.appservice.custom._generic_site_operation', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.custom._get_scm_url', autospec=True)
    @mock.patch('threading.Thread', autospec=True)
    def test_log_stream_supply_cli_ctx(self, threading_mock, get_scm_url_mock, site_op_mock):

        # test exception to exit the streaming loop
        class ErrorToExitInfiniteLoop(Exception):
            pass

        threading_mock.side_effect = ErrorToExitInfiniteLoop('Expected error to exit early')
        get_scm_url_mock.return_value = 'http://great_url'
        cmd_mock = mock.MagicMock()
        cli_ctx_mock = mock.MagicMock()
        cmd_mock.cli_ctx = cli_ctx_mock

        try:
            # action
            get_streaming_log(cmd_mock, 'rg', 'web1')
            self.fail('test exception was not thrown')
        except ErrorToExitInfiniteLoop:
            # assert
            site_op_mock.assert_called_with(cli_ctx_mock, 'rg', 'web1', 'list_publishing_credentials', None)

    @mock.patch('azure.cli.command_modules.appservice.custom._generic_site_operation', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.custom._get_scm_url', autospec=True)
    @mock.patch('azure.cli.command_modules.appservice.custom._get_log', autospec=True)
    def test_download_log_supply_cli_ctx(self, get_log_mock, get_scm_url_mock, site_op_mock):
        def test_result():
            res = mock.MagicMock()
            res.publishing_user_name, res.publishing_password = 'great_user', 'secret_password'
            return res
        test_scm_url = 'http://great_url'
        get_scm_url_mock.return_value = test_scm_url
        publish_cred_mock = mock.MagicMock()
        publish_cred_mock.result = test_result
        site_op_mock.return_value = publish_cred_mock
        cmd_mock = mock.MagicMock()
        cli_ctx_mock = mock.MagicMock()
        cmd_mock.cli_ctx = cli_ctx_mock

        # action
        download_historical_logs(cmd_mock, 'rg', 'web1')

        # assert
        site_op_mock.assert_called_with(cli_ctx_mock, 'rg', 'web1', 'list_publishing_credentials', None)
        get_log_mock.assert_called_with(test_scm_url + '/dump', 'great_user', 'secret_password', None)

    def test_valid_linux_create_options(self):
        some_runtime = 'TOMCAT|8.5-jre8'
        test_docker_image = 'lukasz/great-image:123'
        test_multi_container_config = 'some_config.yaml'
        test_multi_container_type = 'COMPOSE'

        self.assertTrue(validate_linux_create_options(some_runtime, None, None, None))
        self.assertTrue(validate_linux_create_options(None, test_docker_image, None, None))
        self.assertTrue(validate_linux_create_options(None, None, test_multi_container_config, test_multi_container_type))
        self.assertFalse(validate_linux_create_options(some_runtime, None, test_multi_container_config, test_multi_container_type))
        self.assertFalse(validate_linux_create_options(some_runtime, None, test_multi_container_config, None))
        self.assertFalse(validate_linux_create_options(some_runtime, test_docker_image, test_multi_container_config, None))
        self.assertFalse(validate_linux_create_options(None, None, test_multi_container_config, None))
        self.assertFalse(validate_linux_create_options(None, None, None, None))


class FakedResponse(object):  # pylint: disable=too-few-public-methods
    def __init__(self, status_code):
        self.status_code = status_code


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import mock
import unittest
from six import StringIO
from azure.cli.core import AzCommandsLoader
from azure.cli.core.commands import AzCliCommand
from azure.cli.core.parser import AzCliCommandParser

from azure.cli.testsdk import TestCli

from knack.arguments import enum_choice_list


class TestParser(unittest.TestCase):

    def setUp(self):
        self.io = StringIO()

    def tearDown(self):
        self.io.close()

    def test_register_simple_commands(self):
        def test_handler1():
            pass

        def test_handler2():
            pass

        cli = TestCli()
        cli.loader = mock.MagicMock()
        cli.loader.cli_ctx = cli

        command = AzCliCommand(cli.loader, 'command the-name', test_handler1)
        command2 = AzCliCommand(cli.loader, 'sub-command the-second-name', test_handler2)
        cmd_table = {'command the-name': command, 'sub-command the-second-name': command2}

        parser = AzCliCommandParser(cli)
        parser.load_command_table(cmd_table)
        args = parser.parse_args('command the-name'.split())
        self.assertIs(args.func, command)

        args = parser.parse_args('sub-command the-second-name'.split())
        self.assertIs(args.func, command2)

        AzCliCommandParser.error = VerifyError(self,)
        parser.parse_args('sub-command'.split())
        self.assertTrue(AzCliCommandParser.error.called)

    def test_required_parameter(self):
        def test_handler(args):  # pylint: disable=unused-argument
            pass

        cli = TestCli()
        cli.loader = mock.MagicMock()
        cli.loader.cli_ctx = cli

        command = AzCliCommand(cli.loader, 'test command', test_handler)
        command.add_argument('req', '--req', required=True)
        cmd_table = {'test command': command}

        parser = AzCliCommandParser(cli)
        parser.load_command_table(cmd_table)

        args = parser.parse_args('test command --req yep'.split())
        self.assertIs(args.func, command)

        AzCliCommandParser.error = VerifyError(self)
        parser.parse_args('test command'.split())
        self.assertTrue(AzCliCommandParser.error.called)

    def test_nargs_parameter(self):
        def test_handler():
            pass

        cli = TestCli()
        cli.loader = mock.MagicMock()
        cli.loader.cli_ctx = cli

        command = AzCliCommand(cli.loader, 'test command', test_handler)
        command.add_argument('req', '--req', required=True, nargs=2)
        cmd_table = {'test command': command}

        parser = AzCliCommandParser(cli)
        parser.load_command_table(cmd_table)

        args = parser.parse_args('test command --req yep nope'.split())
        self.assertIs(args.func, command)

        AzCliCommandParser.error = VerifyError(self)
        parser.parse_args('test command -req yep'.split())
        self.assertTrue(AzCliCommandParser.error.called)

    def test_case_insensitive_enum_choices(self):
        from enum import Enum

        class TestEnum(Enum):  # pylint: disable=too-few-public-methods

            opt1 = "ALL_CAPS"
            opt2 = "camelCase"
            opt3 = "snake_case"

        def test_handler():
            pass

        cli = TestCli()
        cli.loader = mock.MagicMock()
        cli.loader.cli_ctx = cli

        command = AzCliCommand(cli.loader, 'test command', test_handler)
        command.add_argument('opt', '--opt', required=True, **enum_choice_list(TestEnum))
        cmd_table = {'test command': command}

        parser = AzCliCommandParser(cli)
        parser.load_command_table(cmd_table)

        args = parser.parse_args('test command --opt alL_cAps'.split())
        self.assertEqual(args.opt, 'ALL_CAPS')

        args = parser.parse_args('test command --opt CAMELCASE'.split())
        self.assertEqual(args.opt, 'camelCase')

        args = parser.parse_args('test command --opt sNake_CASE'.split())
        self.assertEqual(args.opt, 'snake_case')


class VerifyError(object):  # pylint: disable=too-few-public-methods

    def __init__(self, test, substr=None):
        self.test = test
        self.substr = substr
        self.called = False

    def __call__(self, message):
        if self.substr:
            self.test.assertTrue(message.find(self.substr) >= 0)
        self.called = True


if __name__ == '__main__':
    unittest.main()
